{"changed":true,"filter":false,"title":"README.md","tooltip":"/README.md","value":"# Memoizerific.js\nFastest (see benchmarks), smallest (923b min/gzip), most-efficient, dependency-free, JavaScript (JS) memoization lib. \nFully supports multiple complex object arguments. \nImplements LRU (least recently used) cache to keep the most recently used results up to the provided limit. \nFor the browser as well as node.\n\nMemoization is the process of caching function results to be returned cheaply when the same arguments are used to call the function again. \n\n## Install\n```javascript\nnpm install memoizerific --save\n```\n\n## Use\n```javascript\nvar memoizerific = require('memoizerific');\n\nvar myExpensiveFunctionMemoized = memoizerific(50)(function(arg1, arg2, arg3) {\n    // so many long expensive calls in here\n});\n\nmyExpensiveFunction(1, 2, 3); // damn, that took looooong to process\nmyExpensiveFunction(1, 2, 3); // wow, that one was instant!\nmyExpensiveFunction(2, 3, 4); // expensive again :(\nmyExpensiveFunction(2, 3, 4); // woah, this one was dirt cheap, I'll take 2!\n```\n\n## Options\nThere is one option available: the max number of results to cache. \n```javascript\nmemoizerific(limit)(fn);\n\nmemoizerific(1)(function(){}); // cache 1 result\nmemoizerific(10000)(function(){}); // cache 10,000 results\nmemoizerific(0)(function(){}); // cache infinity results (not recommended)\n```\nThe cache works using LRU logic (least recently used), purging the oldest results when the limit is reached.\n\n```javascript\n// memoize 1 result\nvar myMemoized = memoizerific(1)(function(arg1, arg2, arg3, arg4) {});\n\nmyMemoized(1, 2, 3, 'a'); // function runs\nmyMemoized(1, 2, 3, 'a'); // cached result is returned\nmyMemoized(1, 2, 3, 'X'); // function runs again, new result is cached, old cached result is purged\nmyMemoized(1, 2, 3, 'X'); // new cached result is returned\nmyMemoized(1, 2, 3, 'a'); // function runs again...\n```\n## Benchmarks\n\nThis library was built for real-world use-cases, we will not be using the easy one-argument fibonacci test commonly used for memoization.\nInstead we will test with multiple complex arguments, that look something like this:\n```javascript\nmyMemoized(\n    { a: 1, b: 2}, // complex object argument\n    [{ x: 'x', q: 'q', }, { b: 8, c: 9 }], // array argument\n    { z: 'z' }, \n    ...\n);\n\n```\n\nWe generate a list of varying numbers of complex arguments (1 to 8 arguments), with varying degrees of variance, to increase and decrease cache hits and misses, \nthen run each memoization lib through them, timing the results. \nFor full details see the source at the [memoize-js-libs-benchmarks](https://github.com/thinkloop/memoize-js-libs-benchmarks) project.\n\nHere are results of some top projects:\n```\n// low variance, mostly cache hits:\n\nno memoization: \nmemoizerific:\nlodash:\nerikras:\nneilk:\n\n// medium variance, lots of cache hits:\n\nno memoization: \nmemoizerific:\nlodash:\nerikras:\nneilk:\n\n// high variance, few cache hits:\n\nno memoization: \nmemoizerific:\nlodash:\nerikras:\nneilk:","undoManager":{"mark":982,"position":1000,"stack":[[{"start":{"row":7,"column":139},"end":{"row":8,"column":0},"action":"insert","lines":["",""],"id":304}],[{"start":{"row":8,"column":0},"end":{"row":8,"column":1},"action":"insert","lines":["A"],"id":305}],[{"start":{"row":8,"column":1},"end":{"row":8,"column":2},"action":"insert","lines":["n"],"id":306}],[{"start":{"row":8,"column":2},"end":{"row":8,"column":3},"action":"insert","lines":["y"],"id":307}],[{"start":{"row":8,"column":2},"end":{"row":8,"column":3},"action":"remove","lines":["y"],"id":308}],[{"start":{"row":8,"column":1},"end":{"row":8,"column":2},"action":"remove","lines":["n"],"id":309}],[{"start":{"row":8,"column":0},"end":{"row":8,"column":1},"action":"remove","lines":["A"],"id":310}],[{"start":{"row":7,"column":139},"end":{"row":8,"column":0},"action":"remove","lines":["",""],"id":311}],[{"start":{"row":1,"column":92},"end":{"row":1,"column":103},"action":"remove","lines":["memoization"],"id":312}],[{"start":{"row":1,"column":91},"end":{"row":1,"column":92},"action":"remove","lines":[" "],"id":313}],[{"start":{"row":1,"column":96},"end":{"row":1,"column":97},"action":"remove","lines":[" "],"id":314}],[{"start":{"row":1,"column":95},"end":{"row":1,"column":96},"action":"remove","lines":["."],"id":315}],[{"start":{"row":1,"column":95},"end":{"row":1,"column":96},"action":"insert","lines":[" "],"id":316}],[{"start":{"row":1,"column":96},"end":{"row":1,"column":97},"action":"insert","lines":["t"],"id":317}],[{"start":{"row":1,"column":97},"end":{"row":1,"column":98},"action":"insert","lines":["o"],"id":318}],[{"start":{"row":1,"column":98},"end":{"row":1,"column":99},"action":"insert","lines":[" "],"id":319}],[{"start":{"row":1,"column":99},"end":{"row":1,"column":100},"action":"insert","lines":["m"],"id":320}],[{"start":{"row":1,"column":100},"end":{"row":1,"column":101},"action":"insert","lines":["e"],"id":321}],[{"start":{"row":1,"column":101},"end":{"row":1,"column":102},"action":"insert","lines":["m"],"id":322}],[{"start":{"row":1,"column":102},"end":{"row":1,"column":103},"action":"insert","lines":["o"],"id":323}],[{"start":{"row":1,"column":103},"end":{"row":1,"column":104},"action":"insert","lines":["i"],"id":324}],[{"start":{"row":1,"column":104},"end":{"row":1,"column":105},"action":"insert","lines":["z"],"id":325}],[{"start":{"row":1,"column":105},"end":{"row":1,"column":106},"action":"insert","lines":["e"],"id":326}],[{"start":{"row":1,"column":106},"end":{"row":1,"column":107},"action":"insert","lines":[" "],"id":327}],[{"start":{"row":1,"column":107},"end":{"row":1,"column":108},"action":"insert","lines":["a"],"id":328}],[{"start":{"row":1,"column":108},"end":{"row":1,"column":109},"action":"insert","lines":["n"],"id":329}],[{"start":{"row":1,"column":109},"end":{"row":1,"column":110},"action":"insert","lines":["y"],"id":330}],[{"start":{"row":1,"column":110},"end":{"row":1,"column":111},"action":"insert","lines":[" "],"id":331}],[{"start":{"row":1,"column":111},"end":{"row":1,"column":112},"action":"insert","lines":["f"],"id":332}],[{"start":{"row":1,"column":112},"end":{"row":1,"column":113},"action":"insert","lines":["u"],"id":333}],[{"start":{"row":1,"column":113},"end":{"row":1,"column":114},"action":"insert","lines":["n"],"id":334}],[{"start":{"row":1,"column":114},"end":{"row":1,"column":115},"action":"insert","lines":["c"],"id":335}],[{"start":{"row":1,"column":115},"end":{"row":1,"column":116},"action":"insert","lines":["t"],"id":336}],[{"start":{"row":1,"column":116},"end":{"row":1,"column":117},"action":"insert","lines":["i"],"id":337}],[{"start":{"row":1,"column":117},"end":{"row":1,"column":118},"action":"insert","lines":["o"],"id":338}],[{"start":{"row":1,"column":118},"end":{"row":1,"column":119},"action":"insert","lines":["n"],"id":339}],[{"start":{"row":1,"column":119},"end":{"row":1,"column":120},"action":"insert","lines":["."],"id":340}],[{"start":{"row":34,"column":40},"end":{"row":34,"column":48},"action":"remove","lines":["infinite"],"id":341},{"start":{"row":34,"column":40},"end":{"row":34,"column":41},"action":"insert","lines":["i"]}],[{"start":{"row":34,"column":41},"end":{"row":34,"column":42},"action":"insert","lines":["n"],"id":342}],[{"start":{"row":34,"column":42},"end":{"row":34,"column":43},"action":"insert","lines":["f"],"id":343}],[{"start":{"row":34,"column":43},"end":{"row":34,"column":44},"action":"insert","lines":["i"],"id":344}],[{"start":{"row":34,"column":44},"end":{"row":34,"column":45},"action":"insert","lines":["n"],"id":345}],[{"start":{"row":34,"column":45},"end":{"row":34,"column":46},"action":"insert","lines":["i"],"id":346}],[{"start":{"row":34,"column":46},"end":{"row":34,"column":47},"action":"insert","lines":["t"],"id":347}],[{"start":{"row":34,"column":47},"end":{"row":34,"column":48},"action":"insert","lines":["y"],"id":348}],[{"start":{"row":34,"column":56},"end":{"row":34,"column":57},"action":"insert","lines":[" "],"id":349}],[{"start":{"row":34,"column":57},"end":{"row":34,"column":58},"action":"insert","lines":["("],"id":350}],[{"start":{"row":34,"column":58},"end":{"row":34,"column":59},"action":"insert","lines":["n"],"id":351}],[{"start":{"row":34,"column":59},"end":{"row":34,"column":60},"action":"insert","lines":["o"],"id":352}],[{"start":{"row":34,"column":60},"end":{"row":34,"column":61},"action":"insert","lines":["t"],"id":353}],[{"start":{"row":34,"column":61},"end":{"row":34,"column":62},"action":"insert","lines":[" "],"id":354}],[{"start":{"row":34,"column":62},"end":{"row":34,"column":63},"action":"insert","lines":["r"],"id":355}],[{"start":{"row":34,"column":63},"end":{"row":34,"column":64},"action":"insert","lines":["e"],"id":356}],[{"start":{"row":34,"column":64},"end":{"row":34,"column":65},"action":"insert","lines":["c"],"id":357}],[{"start":{"row":34,"column":65},"end":{"row":34,"column":66},"action":"insert","lines":["o"],"id":358}],[{"start":{"row":34,"column":66},"end":{"row":34,"column":67},"action":"insert","lines":["m"],"id":359}],[{"start":{"row":34,"column":67},"end":{"row":34,"column":68},"action":"insert","lines":["m"],"id":360}],[{"start":{"row":34,"column":68},"end":{"row":34,"column":69},"action":"insert","lines":["e"],"id":361}],[{"start":{"row":34,"column":69},"end":{"row":34,"column":70},"action":"insert","lines":["n"],"id":362}],[{"start":{"row":34,"column":70},"end":{"row":34,"column":71},"action":"insert","lines":["d"],"id":363}],[{"start":{"row":34,"column":71},"end":{"row":34,"column":72},"action":"insert","lines":["e"],"id":364}],[{"start":{"row":34,"column":72},"end":{"row":34,"column":73},"action":"insert","lines":["d"],"id":365}],[{"start":{"row":34,"column":73},"end":{"row":34,"column":74},"action":"insert","lines":[")"],"id":366}],[{"start":{"row":31,"column":24},"end":{"row":32,"column":0},"action":"insert","lines":["",""],"id":367}],[{"start":{"row":40,"column":11},"end":{"row":40,"column":15},"action":"remove","lines":["only"],"id":368}],[{"start":{"row":40,"column":10},"end":{"row":40,"column":11},"action":"remove","lines":[" "],"id":369}],[{"start":{"row":40,"column":15},"end":{"row":40,"column":16},"action":"insert","lines":["s"],"id":370}],[{"start":{"row":41,"column":84},"end":{"row":43,"column":0},"action":"remove","lines":["","    // so many long expensive calls in here",""],"id":371}],[{"start":{"row":45,"column":78},"end":{"row":45,"column":79},"action":"insert","lines":["i"],"id":372}],[{"start":{"row":45,"column":79},"end":{"row":45,"column":80},"action":"insert","lines":["s"],"id":373}],[{"start":{"row":45,"column":80},"end":{"row":45,"column":81},"action":"insert","lines":[" "],"id":374}],[{"start":{"row":45,"column":107},"end":{"row":45,"column":108},"action":"insert","lines":["i"],"id":375}],[{"start":{"row":45,"column":108},"end":{"row":45,"column":109},"action":"insert","lines":["s"],"id":376}],[{"start":{"row":45,"column":109},"end":{"row":45,"column":110},"action":"insert","lines":[" "],"id":377}],[{"start":{"row":50,"column":0},"end":{"row":54,"column":3},"action":"remove","lines":["We can also have an unlimited cache size (not recommended):","```javascript","var myExpensiveFunctionMemoized = memoizerific(0)(function(arg1, arg2, arg3) {","    // so many long expensive calls in here","});"],"id":378}],[{"start":{"row":49,"column":3},"end":{"row":50,"column":0},"action":"remove","lines":["",""],"id":379}],[{"start":{"row":49,"column":2},"end":{"row":49,"column":3},"action":"remove","lines":["`"],"id":380}],[{"start":{"row":49,"column":1},"end":{"row":49,"column":2},"action":"remove","lines":["`"],"id":381}],[{"start":{"row":49,"column":0},"end":{"row":49,"column":1},"action":"remove","lines":["`"],"id":382}],[{"start":{"row":48,"column":0},"end":{"row":49,"column":0},"action":"remove","lines":["",""],"id":383}],[{"start":{"row":47,"column":68},"end":{"row":48,"column":0},"action":"remove","lines":["",""],"id":384}],[{"start":{"row":51,"column":65},"end":{"row":51,"column":73},"action":"remove","lines":["standard"],"id":385},{"start":{"row":51,"column":65},"end":{"row":51,"column":66},"action":"insert","lines":["u"]}],[{"start":{"row":51,"column":66},"end":{"row":51,"column":67},"action":"insert","lines":["s"],"id":386}],[{"start":{"row":51,"column":67},"end":{"row":51,"column":68},"action":"insert","lines":["u"],"id":387}],[{"start":{"row":51,"column":68},"end":{"row":51,"column":69},"action":"insert","lines":["a"],"id":388}],[{"start":{"row":51,"column":69},"end":{"row":51,"column":70},"action":"insert","lines":["l"],"id":389}],[{"start":{"row":51,"column":23},"end":{"row":51,"column":27},"action":"remove","lines":["with"],"id":390},{"start":{"row":51,"column":23},"end":{"row":51,"column":24},"action":"insert","lines":["f"]}],[{"start":{"row":51,"column":24},"end":{"row":51,"column":25},"action":"insert","lines":["o"],"id":391}],[{"start":{"row":51,"column":25},"end":{"row":51,"column":26},"action":"insert","lines":["r"],"id":392}],[{"start":{"row":51,"column":47},"end":{"row":51,"column":55},"action":"remove","lines":[" in mind"],"id":393}],[{"start":{"row":51,"column":52},"end":{"row":51,"column":53},"action":"insert","lines":["w"],"id":394}],[{"start":{"row":51,"column":53},"end":{"row":51,"column":54},"action":"insert","lines":["e"],"id":395}],[{"start":{"row":51,"column":54},"end":{"row":51,"column":55},"action":"insert","lines":[" "],"id":396}],[{"start":{"row":51,"column":55},"end":{"row":51,"column":56},"action":"insert","lines":["w"],"id":397}],[{"start":{"row":51,"column":56},"end":{"row":51,"column":57},"action":"insert","lines":["i"],"id":398}],[{"start":{"row":51,"column":57},"end":{"row":51,"column":58},"action":"insert","lines":["l"],"id":399}],[{"start":{"row":51,"column":58},"end":{"row":51,"column":59},"action":"insert","lines":["l"],"id":400}],[{"start":{"row":51,"column":59},"end":{"row":51,"column":60},"action":"insert","lines":[" "],"id":401}],[{"start":{"row":51,"column":60},"end":{"row":51,"column":61},"action":"insert","lines":["n"],"id":402}],[{"start":{"row":51,"column":61},"end":{"row":51,"column":62},"action":"insert","lines":["o"],"id":403}],[{"start":{"row":51,"column":62},"end":{"row":51,"column":63},"action":"insert","lines":["t"],"id":404}],[{"start":{"row":51,"column":63},"end":{"row":51,"column":64},"action":"insert","lines":[" "],"id":405}],[{"start":{"row":51,"column":64},"end":{"row":51,"column":65},"action":"insert","lines":["b"],"id":406}],[{"start":{"row":51,"column":65},"end":{"row":51,"column":66},"action":"insert","lines":["e"],"id":407}],[{"start":{"row":51,"column":66},"end":{"row":51,"column":67},"action":"insert","lines":[" "],"id":408}],[{"start":{"row":51,"column":67},"end":{"row":51,"column":68},"action":"insert","lines":["u"],"id":409}],[{"start":{"row":51,"column":68},"end":{"row":51,"column":69},"action":"insert","lines":["s"],"id":410}],[{"start":{"row":51,"column":69},"end":{"row":51,"column":70},"action":"insert","lines":["i"],"id":411}],[{"start":{"row":51,"column":70},"end":{"row":51,"column":71},"action":"insert","lines":["n"],"id":412}],[{"start":{"row":51,"column":71},"end":{"row":51,"column":72},"action":"insert","lines":["g"],"id":413}],[{"start":{"row":51,"column":72},"end":{"row":51,"column":73},"action":"insert","lines":[" "],"id":414}],[{"start":{"row":51,"column":110},"end":{"row":51,"column":111},"action":"insert","lines":[","],"id":415}],[{"start":{"row":51,"column":111},"end":{"row":51,"column":112},"action":"insert","lines":[" "],"id":416}],[{"start":{"row":51,"column":112},"end":{"row":51,"column":113},"action":"insert","lines":["i"],"id":417}],[{"start":{"row":51,"column":113},"end":{"row":51,"column":114},"action":"insert","lines":["n"],"id":418}],[{"start":{"row":51,"column":114},"end":{"row":51,"column":115},"action":"insert","lines":["s"],"id":419}],[{"start":{"row":51,"column":115},"end":{"row":51,"column":116},"action":"insert","lines":["t"],"id":420}],[{"start":{"row":51,"column":116},"end":{"row":51,"column":117},"action":"insert","lines":["e"],"id":421}],[{"start":{"row":51,"column":117},"end":{"row":51,"column":118},"action":"insert","lines":["a"],"id":422}],[{"start":{"row":51,"column":118},"end":{"row":51,"column":119},"action":"insert","lines":["d"],"id":423}],[{"start":{"row":51,"column":119},"end":{"row":51,"column":120},"action":"insert","lines":[" "],"id":424}],[{"start":{"row":51,"column":120},"end":{"row":51,"column":121},"action":"insert","lines":["o"],"id":425}],[{"start":{"row":51,"column":121},"end":{"row":51,"column":122},"action":"insert","lines":["p"],"id":426}],[{"start":{"row":51,"column":122},"end":{"row":51,"column":123},"action":"insert","lines":["t"],"id":427}],[{"start":{"row":51,"column":123},"end":{"row":51,"column":124},"action":"insert","lines":["i"],"id":428}],[{"start":{"row":51,"column":124},"end":{"row":51,"column":125},"action":"insert","lines":["n"],"id":429}],[{"start":{"row":51,"column":125},"end":{"row":51,"column":126},"action":"insert","lines":["g"],"id":430}],[{"start":{"row":51,"column":110},"end":{"row":51,"column":111},"action":"insert","lines":["."],"id":431}],[{"start":{"row":51,"column":111},"end":{"row":51,"column":151},"action":"remove","lines":[", instead opting case will not be used. "],"id":432}],[{"start":{"row":51,"column":106},"end":{"row":51,"column":110},"action":"remove","lines":["test"],"id":433},{"start":{"row":51,"column":106},"end":{"row":51,"column":107},"action":"insert","lines":["f"]}],[{"start":{"row":51,"column":107},"end":{"row":51,"column":108},"action":"insert","lines":["u"],"id":434}],[{"start":{"row":51,"column":108},"end":{"row":51,"column":109},"action":"insert","lines":["n"],"id":435}],[{"start":{"row":51,"column":109},"end":{"row":51,"column":110},"action":"insert","lines":["c"],"id":436}],[{"start":{"row":51,"column":110},"end":{"row":51,"column":111},"action":"insert","lines":["t"],"id":437}],[{"start":{"row":51,"column":111},"end":{"row":51,"column":112},"action":"insert","lines":["i"],"id":438}],[{"start":{"row":51,"column":112},"end":{"row":51,"column":113},"action":"insert","lines":["o"],"id":439}],[{"start":{"row":51,"column":113},"end":{"row":51,"column":114},"action":"insert","lines":["n"],"id":440}],[{"start":{"row":51,"column":77},"end":{"row":51,"column":82},"action":"remove","lines":["usual"],"id":441},{"start":{"row":51,"column":77},"end":{"row":51,"column":78},"action":"insert","lines":["b"]}],[{"start":{"row":51,"column":78},"end":{"row":51,"column":79},"action":"insert","lines":["a"],"id":442}],[{"start":{"row":51,"column":79},"end":{"row":51,"column":80},"action":"insert","lines":["s"],"id":443}],[{"start":{"row":51,"column":80},"end":{"row":51,"column":81},"action":"insert","lines":["i"],"id":444}],[{"start":{"row":51,"column":81},"end":{"row":51,"column":82},"action":"insert","lines":["c"],"id":445}],[{"start":{"row":51,"column":114},"end":{"row":51,"column":115},"action":"insert","lines":[" "],"id":446}],[{"start":{"row":51,"column":115},"end":{"row":51,"column":116},"action":"insert","lines":["t"],"id":447}],[{"start":{"row":51,"column":116},"end":{"row":51,"column":117},"action":"insert","lines":["h"],"id":448}],[{"start":{"row":51,"column":117},"end":{"row":51,"column":118},"action":"insert","lines":["a"],"id":449}],[{"start":{"row":51,"column":118},"end":{"row":51,"column":119},"action":"insert","lines":["t"],"id":450}],[{"start":{"row":51,"column":119},"end":{"row":51,"column":120},"action":"insert","lines":[" "],"id":451}],[{"start":{"row":51,"column":120},"end":{"row":51,"column":121},"action":"insert","lines":["i"],"id":452}],[{"start":{"row":51,"column":121},"end":{"row":51,"column":122},"action":"insert","lines":["s"],"id":453}],[{"start":{"row":51,"column":122},"end":{"row":51,"column":123},"action":"insert","lines":[" "],"id":454}],[{"start":{"row":51,"column":123},"end":{"row":51,"column":124},"action":"insert","lines":["o"],"id":455}],[{"start":{"row":51,"column":124},"end":{"row":51,"column":125},"action":"insert","lines":["f"],"id":456}],[{"start":{"row":51,"column":125},"end":{"row":51,"column":126},"action":"insert","lines":["t"],"id":457}],[{"start":{"row":51,"column":126},"end":{"row":51,"column":127},"action":"insert","lines":["e"],"id":458}],[{"start":{"row":51,"column":127},"end":{"row":51,"column":128},"action":"insert","lines":["n"],"id":459}],[{"start":{"row":51,"column":128},"end":{"row":51,"column":129},"action":"insert","lines":[" "],"id":460}],[{"start":{"row":51,"column":129},"end":{"row":51,"column":130},"action":"insert","lines":["u"],"id":461}],[{"start":{"row":51,"column":130},"end":{"row":51,"column":131},"action":"insert","lines":["s"],"id":462}],[{"start":{"row":51,"column":131},"end":{"row":51,"column":132},"action":"insert","lines":["e"],"id":463}],[{"start":{"row":51,"column":132},"end":{"row":51,"column":133},"action":"insert","lines":["d"],"id":464}],[{"start":{"row":51,"column":133},"end":{"row":51,"column":134},"action":"insert","lines":[" "],"id":465}],[{"start":{"row":51,"column":134},"end":{"row":51,"column":135},"action":"insert","lines":["t"],"id":466}],[{"start":{"row":51,"column":135},"end":{"row":51,"column":136},"action":"insert","lines":["o"],"id":467}],[{"start":{"row":51,"column":136},"end":{"row":51,"column":137},"action":"insert","lines":[" "],"id":468}],[{"start":{"row":51,"column":137},"end":{"row":51,"column":138},"action":"insert","lines":["t"],"id":469}],[{"start":{"row":51,"column":138},"end":{"row":51,"column":139},"action":"insert","lines":["r"],"id":470}],[{"start":{"row":51,"column":139},"end":{"row":51,"column":140},"action":"insert","lines":["s"],"id":471}],[{"start":{"row":51,"column":139},"end":{"row":51,"column":140},"action":"remove","lines":["s"],"id":472}],[{"start":{"row":51,"column":138},"end":{"row":51,"column":139},"action":"remove","lines":["r"],"id":473}],[{"start":{"row":51,"column":138},"end":{"row":51,"column":139},"action":"insert","lines":["e"],"id":474}],[{"start":{"row":51,"column":139},"end":{"row":51,"column":140},"action":"insert","lines":["s"],"id":475}],[{"start":{"row":51,"column":140},"end":{"row":51,"column":141},"action":"insert","lines":["t"],"id":476}],[{"start":{"row":51,"column":141},"end":{"row":51,"column":142},"action":"insert","lines":[" "],"id":477}],[{"start":{"row":51,"column":142},"end":{"row":51,"column":143},"action":"insert","lines":["m"],"id":478}],[{"start":{"row":51,"column":143},"end":{"row":51,"column":144},"action":"insert","lines":["e"],"id":479}],[{"start":{"row":51,"column":144},"end":{"row":51,"column":145},"action":"insert","lines":["m"],"id":480}],[{"start":{"row":51,"column":145},"end":{"row":51,"column":146},"action":"insert","lines":["o"],"id":481}],[{"start":{"row":51,"column":146},"end":{"row":51,"column":147},"action":"insert","lines":["i"],"id":482}],[{"start":{"row":51,"column":147},"end":{"row":51,"column":148},"action":"insert","lines":["z"],"id":483}],[{"start":{"row":51,"column":148},"end":{"row":51,"column":149},"action":"insert","lines":["a"],"id":484}],[{"start":{"row":51,"column":149},"end":{"row":51,"column":150},"action":"insert","lines":["t"],"id":485}],[{"start":{"row":51,"column":150},"end":{"row":51,"column":151},"action":"insert","lines":["i"],"id":486}],[{"start":{"row":51,"column":151},"end":{"row":51,"column":152},"action":"insert","lines":["o"],"id":487}],[{"start":{"row":51,"column":152},"end":{"row":51,"column":153},"action":"insert","lines":["n"],"id":488}],[{"start":{"row":51,"column":153},"end":{"row":51,"column":154},"action":"insert","lines":[" "],"id":489}],[{"start":{"row":51,"column":154},"end":{"row":51,"column":155},"action":"insert","lines":["l"],"id":490}],[{"start":{"row":51,"column":155},"end":{"row":51,"column":156},"action":"insert","lines":["i"],"id":491}],[{"start":{"row":51,"column":156},"end":{"row":51,"column":157},"action":"insert","lines":["b"],"id":492}],[{"start":{"row":51,"column":157},"end":{"row":51,"column":158},"action":"insert","lines":["s"],"id":493}],[{"start":{"row":52,"column":11},"end":{"row":52,"column":12},"action":"insert","lines":["w"],"id":494}],[{"start":{"row":52,"column":12},"end":{"row":52,"column":13},"action":"insert","lines":["l"],"id":495}],[{"start":{"row":52,"column":13},"end":{"row":52,"column":14},"action":"insert","lines":["l"],"id":496}],[{"start":{"row":52,"column":14},"end":{"row":52,"column":15},"action":"insert","lines":[" "],"id":497}],[{"start":{"row":52,"column":15},"end":{"row":52,"column":16},"action":"insert","lines":["b"],"id":498}],[{"start":{"row":52,"column":16},"end":{"row":52,"column":17},"action":"insert","lines":["e"],"id":499}],[{"start":{"row":52,"column":17},"end":{"row":52,"column":18},"action":"insert","lines":[" "],"id":500}],[{"start":{"row":52,"column":12},"end":{"row":52,"column":13},"action":"insert","lines":["i"],"id":501}],[{"start":{"row":52,"column":23},"end":{"row":52,"column":24},"action":"insert","lines":["i"],"id":502}],[{"start":{"row":52,"column":24},"end":{"row":52,"column":25},"action":"insert","lines":["n"],"id":503}],[{"start":{"row":52,"column":25},"end":{"row":52,"column":26},"action":"insert","lines":["g"],"id":504}],[{"start":{"row":52,"column":59},"end":{"row":52,"column":60},"action":"insert","lines":["t"],"id":505}],[{"start":{"row":52,"column":60},"end":{"row":52,"column":61},"action":"insert","lines":["h"],"id":506}],[{"start":{"row":52,"column":61},"end":{"row":52,"column":62},"action":"insert","lines":["a"],"id":507}],[{"start":{"row":52,"column":62},"end":{"row":52,"column":63},"action":"insert","lines":["t"],"id":508}],[{"start":{"row":52,"column":63},"end":{"row":52,"column":64},"action":"insert","lines":[" "],"id":509}],[{"start":{"row":52,"column":64},"end":{"row":52,"column":65},"action":"insert","lines":["l"],"id":510}],[{"start":{"row":52,"column":65},"end":{"row":52,"column":66},"action":"insert","lines":["o"],"id":511}],[{"start":{"row":52,"column":66},"end":{"row":52,"column":67},"action":"insert","lines":["o"],"id":512}],[{"start":{"row":52,"column":67},"end":{"row":52,"column":68},"action":"insert","lines":["k"],"id":513}],[{"start":{"row":52,"column":68},"end":{"row":52,"column":69},"action":"insert","lines":[" "],"id":514}],[{"start":{"row":52,"column":69},"end":{"row":52,"column":70},"action":"insert","lines":["s"],"id":515}],[{"start":{"row":52,"column":70},"end":{"row":52,"column":71},"action":"insert","lines":["o"],"id":516}],[{"start":{"row":52,"column":71},"end":{"row":52,"column":72},"action":"insert","lines":["m"],"id":517}],[{"start":{"row":52,"column":72},"end":{"row":52,"column":73},"action":"insert","lines":["e"],"id":518}],[{"start":{"row":52,"column":73},"end":{"row":52,"column":74},"action":"insert","lines":["t"],"id":519}],[{"start":{"row":52,"column":74},"end":{"row":52,"column":75},"action":"insert","lines":["h"],"id":520}],[{"start":{"row":52,"column":75},"end":{"row":52,"column":76},"action":"insert","lines":["i"],"id":521}],[{"start":{"row":52,"column":76},"end":{"row":52,"column":77},"action":"insert","lines":["n"],"id":522}],[{"start":{"row":52,"column":77},"end":{"row":52,"column":78},"action":"insert","lines":["g"],"id":523}],[{"start":{"row":52,"column":78},"end":{"row":52,"column":79},"action":"insert","lines":[" "],"id":524}],[{"start":{"row":54,"column":28},"end":{"row":55,"column":0},"action":"insert","lines":["",""],"id":525}],[{"start":{"row":55,"column":0},"end":{"row":55,"column":4},"action":"insert","lines":["    "],"id":526}],[{"start":{"row":55,"column":19},"end":{"row":55,"column":20},"action":"insert","lines":["'"],"id":527},{"start":{"row":55,"column":20},"end":{"row":56,"column":0},"action":"insert","lines":["",""]},{"start":{"row":56,"column":0},"end":{"row":56,"column":4},"action":"insert","lines":["    "]}],[{"start":{"row":56,"column":0},"end":{"row":56,"column":4},"action":"remove","lines":["    "],"id":528}],[{"start":{"row":55,"column":20},"end":{"row":56,"column":0},"action":"remove","lines":["",""],"id":529}],[{"start":{"row":55,"column":19},"end":{"row":55,"column":20},"action":"remove","lines":["'"],"id":530}],[{"start":{"row":55,"column":19},"end":{"row":56,"column":0},"action":"insert","lines":["",""],"id":531},{"start":{"row":56,"column":0},"end":{"row":56,"column":4},"action":"insert","lines":["    "]}],[{"start":{"row":56,"column":43},"end":{"row":57,"column":0},"action":"insert","lines":["",""],"id":532},{"start":{"row":57,"column":0},"end":{"row":57,"column":4},"action":"insert","lines":["    "]}],[{"start":{"row":57,"column":16},"end":{"row":58,"column":0},"action":"insert","lines":["",""],"id":533},{"start":{"row":58,"column":0},"end":{"row":58,"column":4},"action":"insert","lines":["    "]}],[{"start":{"row":58,"column":7},"end":{"row":59,"column":0},"action":"insert","lines":["",""],"id":534},{"start":{"row":59,"column":0},"end":{"row":59,"column":4},"action":"insert","lines":["    "]}],[{"start":{"row":59,"column":0},"end":{"row":59,"column":4},"action":"remove","lines":["    "],"id":535}],[{"start":{"row":55,"column":19},"end":{"row":55,"column":20},"action":"insert","lines":["/"],"id":536}],[{"start":{"row":55,"column":20},"end":{"row":55,"column":21},"action":"insert","lines":["/"],"id":537}],[{"start":{"row":55,"column":21},"end":{"row":55,"column":22},"action":"insert","lines":[" "],"id":538}],[{"start":{"row":55,"column":22},"end":{"row":55,"column":23},"action":"insert","lines":["o"],"id":539}],[{"start":{"row":55,"column":23},"end":{"row":55,"column":24},"action":"insert","lines":["b"],"id":540}],[{"start":{"row":55,"column":24},"end":{"row":55,"column":25},"action":"insert","lines":["j"],"id":541}],[{"start":{"row":55,"column":25},"end":{"row":55,"column":26},"action":"insert","lines":["e"],"id":542}],[{"start":{"row":55,"column":26},"end":{"row":55,"column":27},"action":"insert","lines":["c"],"id":543}],[{"start":{"row":55,"column":27},"end":{"row":55,"column":28},"action":"insert","lines":["t"],"id":544}],[{"start":{"row":55,"column":28},"end":{"row":55,"column":29},"action":"insert","lines":[" "],"id":545}],[{"start":{"row":55,"column":29},"end":{"row":55,"column":30},"action":"insert","lines":["a"],"id":546}],[{"start":{"row":55,"column":30},"end":{"row":55,"column":31},"action":"insert","lines":["r"],"id":547}],[{"start":{"row":55,"column":31},"end":{"row":55,"column":32},"action":"insert","lines":["g"],"id":548}],[{"start":{"row":55,"column":32},"end":{"row":55,"column":33},"action":"insert","lines":["u"],"id":549}],[{"start":{"row":55,"column":33},"end":{"row":55,"column":34},"action":"insert","lines":["m"],"id":550}],[{"start":{"row":55,"column":34},"end":{"row":55,"column":35},"action":"insert","lines":["e"],"id":551}],[{"start":{"row":55,"column":35},"end":{"row":55,"column":36},"action":"insert","lines":["n"],"id":552}],[{"start":{"row":55,"column":36},"end":{"row":55,"column":37},"action":"insert","lines":["t"],"id":553}],[{"start":{"row":56,"column":43},"end":{"row":56,"column":44},"action":"insert","lines":["/"],"id":554}],[{"start":{"row":56,"column":44},"end":{"row":56,"column":45},"action":"insert","lines":["/"],"id":555}],[{"start":{"row":56,"column":45},"end":{"row":56,"column":46},"action":"insert","lines":["a"],"id":556}],[{"start":{"row":56,"column":46},"end":{"row":56,"column":47},"action":"insert","lines":["r"],"id":557}],[{"start":{"row":56,"column":47},"end":{"row":56,"column":48},"action":"insert","lines":["r"],"id":558}],[{"start":{"row":56,"column":48},"end":{"row":56,"column":49},"action":"insert","lines":["a"],"id":559}],[{"start":{"row":56,"column":49},"end":{"row":56,"column":50},"action":"insert","lines":["y"],"id":560}],[{"start":{"row":56,"column":50},"end":{"row":56,"column":51},"action":"insert","lines":[" "],"id":561}],[{"start":{"row":56,"column":51},"end":{"row":56,"column":52},"action":"insert","lines":["a"],"id":562}],[{"start":{"row":56,"column":52},"end":{"row":56,"column":53},"action":"insert","lines":["r"],"id":563}],[{"start":{"row":56,"column":53},"end":{"row":56,"column":54},"action":"insert","lines":["g"],"id":564}],[{"start":{"row":56,"column":54},"end":{"row":56,"column":55},"action":"insert","lines":["u"],"id":565}],[{"start":{"row":56,"column":55},"end":{"row":56,"column":56},"action":"insert","lines":["m"],"id":566}],[{"start":{"row":56,"column":56},"end":{"row":56,"column":57},"action":"insert","lines":["e"],"id":567}],[{"start":{"row":56,"column":57},"end":{"row":56,"column":58},"action":"insert","lines":["n"],"id":568}],[{"start":{"row":56,"column":58},"end":{"row":56,"column":59},"action":"insert","lines":["t"],"id":569}],[{"start":{"row":55,"column":22},"end":{"row":55,"column":23},"action":"insert","lines":["c"],"id":570}],[{"start":{"row":55,"column":23},"end":{"row":55,"column":24},"action":"insert","lines":["o"],"id":571}],[{"start":{"row":55,"column":24},"end":{"row":55,"column":25},"action":"insert","lines":["m"],"id":572}],[{"start":{"row":55,"column":25},"end":{"row":55,"column":26},"action":"insert","lines":["p"],"id":573}],[{"start":{"row":55,"column":26},"end":{"row":55,"column":27},"action":"insert","lines":["l"],"id":574}],[{"start":{"row":55,"column":27},"end":{"row":55,"column":28},"action":"insert","lines":["e"],"id":575}],[{"start":{"row":55,"column":28},"end":{"row":55,"column":29},"action":"insert","lines":["x"],"id":576}],[{"start":{"row":55,"column":29},"end":{"row":55,"column":30},"action":"insert","lines":[" "],"id":577}],[{"start":{"row":56,"column":45},"end":{"row":56,"column":46},"action":"insert","lines":[" "],"id":578}],[{"start":{"row":54,"column":2},"end":{"row":54,"column":19},"action":"remove","lines":["ExpensiveFunction"],"id":579}],[{"start":{"row":43,"column":2},"end":{"row":43,"column":19},"action":"remove","lines":["ExpensiveFunction"],"id":580}],[{"start":{"row":44,"column":2},"end":{"row":44,"column":19},"action":"remove","lines":["ExpensiveFunction"],"id":581}],[{"start":{"row":45,"column":2},"end":{"row":45,"column":19},"action":"remove","lines":["ExpensiveFunction"],"id":582}],[{"start":{"row":46,"column":2},"end":{"row":46,"column":19},"action":"remove","lines":["ExpensiveFunction"],"id":583}],[{"start":{"row":47,"column":2},"end":{"row":47,"column":19},"action":"remove","lines":["ExpensiveFunction"],"id":584}],[{"start":{"row":41,"column":4},"end":{"row":41,"column":31},"action":"remove","lines":["myExpensiveFunctionMemoized"],"id":585},{"start":{"row":41,"column":4},"end":{"row":41,"column":14},"action":"insert","lines":["myMemoized"]}],[{"start":{"row":51,"column":154},"end":{"row":51,"column":158},"action":"remove","lines":["libs"],"id":586}],[{"start":{"row":51,"column":153},"end":{"row":51,"column":154},"action":"remove","lines":[" "],"id":587}],[{"start":{"row":51,"column":77},"end":{"row":51,"column":82},"action":"remove","lines":["basic"],"id":588},{"start":{"row":51,"column":77},"end":{"row":51,"column":78},"action":"insert","lines":["e"]}],[{"start":{"row":51,"column":78},"end":{"row":51,"column":79},"action":"insert","lines":["a"],"id":589}],[{"start":{"row":51,"column":79},"end":{"row":51,"column":80},"action":"insert","lines":["s"],"id":590}],[{"start":{"row":51,"column":80},"end":{"row":51,"column":81},"action":"insert","lines":["y"],"id":591}],[{"start":{"row":51,"column":105},"end":{"row":51,"column":113},"action":"remove","lines":["function"],"id":592},{"start":{"row":51,"column":105},"end":{"row":51,"column":106},"action":"insert","lines":["t"]}],[{"start":{"row":51,"column":106},"end":{"row":51,"column":107},"action":"insert","lines":["e"],"id":593}],[{"start":{"row":51,"column":107},"end":{"row":51,"column":108},"action":"insert","lines":["s"],"id":594}],[{"start":{"row":51,"column":108},"end":{"row":51,"column":109},"action":"insert","lines":["t"],"id":595}],[{"start":{"row":51,"column":129},"end":{"row":51,"column":136},"action":"remove","lines":["to test"],"id":596},{"start":{"row":51,"column":129},"end":{"row":51,"column":130},"action":"insert","lines":["f"]}],[{"start":{"row":51,"column":130},"end":{"row":51,"column":131},"action":"insert","lines":["o"],"id":597}],[{"start":{"row":51,"column":131},"end":{"row":51,"column":132},"action":"insert","lines":["r"],"id":598}],[{"start":{"row":52,"column":16},"end":{"row":52,"column":18},"action":"remove","lines":["be"],"id":599}],[{"start":{"row":52,"column":15},"end":{"row":52,"column":16},"action":"remove","lines":[" "],"id":600}],[{"start":{"row":52,"column":20},"end":{"row":52,"column":23},"action":"remove","lines":["ing"],"id":601}],[{"start":{"row":63,"column":0},"end":{"row":63,"column":51},"action":"remove","lines":["Here are results against some of the top libraries:"],"id":602},{"start":{"row":63,"column":0},"end":{"row":63,"column":1},"action":"insert","lines":["T"]}],[{"start":{"row":63,"column":1},"end":{"row":63,"column":2},"action":"insert","lines":["h"],"id":603}],[{"start":{"row":63,"column":2},"end":{"row":63,"column":3},"action":"insert","lines":["e"],"id":604}],[{"start":{"row":63,"column":3},"end":{"row":63,"column":4},"action":"insert","lines":[" "],"id":605}],[{"start":{"row":63,"column":4},"end":{"row":63,"column":5},"action":"insert","lines":["b"],"id":606}],[{"start":{"row":63,"column":5},"end":{"row":63,"column":6},"action":"insert","lines":["e"],"id":607}],[{"start":{"row":63,"column":6},"end":{"row":63,"column":7},"action":"insert","lines":["n"],"id":608}],[{"start":{"row":63,"column":7},"end":{"row":63,"column":8},"action":"insert","lines":["c"],"id":609}],[{"start":{"row":63,"column":8},"end":{"row":63,"column":9},"action":"insert","lines":["h"],"id":610}],[{"start":{"row":63,"column":9},"end":{"row":63,"column":10},"action":"insert","lines":["m"],"id":611}],[{"start":{"row":63,"column":10},"end":{"row":63,"column":11},"action":"insert","lines":["a"],"id":612}],[{"start":{"row":63,"column":11},"end":{"row":63,"column":12},"action":"insert","lines":["r"],"id":613}],[{"start":{"row":63,"column":12},"end":{"row":63,"column":13},"action":"insert","lines":["k"],"id":614}],[{"start":{"row":63,"column":13},"end":{"row":63,"column":14},"action":"insert","lines":["s"],"id":615}],[{"start":{"row":63,"column":14},"end":{"row":63,"column":15},"action":"insert","lines":[" "],"id":616}],[{"start":{"row":51,"column":49},"end":{"row":51,"column":51},"action":"remove","lines":["so"],"id":617}],[{"start":{"row":51,"column":48},"end":{"row":51,"column":49},"action":"remove","lines":[" "],"id":618}],[{"start":{"row":51,"column":107},"end":{"row":51,"column":111},"action":"remove","lines":["that"],"id":619},{"start":{"row":51,"column":107},"end":{"row":51,"column":108},"action":"insert","lines":["c"]}],[{"start":{"row":51,"column":108},"end":{"row":51,"column":109},"action":"insert","lines":["o"],"id":620}],[{"start":{"row":51,"column":109},"end":{"row":51,"column":110},"action":"insert","lines":["m"],"id":621}],[{"start":{"row":51,"column":110},"end":{"row":51,"column":111},"action":"insert","lines":["m"],"id":622}],[{"start":{"row":51,"column":111},"end":{"row":51,"column":112},"action":"insert","lines":["o"],"id":623}],[{"start":{"row":51,"column":112},"end":{"row":51,"column":113},"action":"insert","lines":["n"],"id":624}],[{"start":{"row":51,"column":113},"end":{"row":51,"column":114},"action":"insert","lines":["l"],"id":625}],[{"start":{"row":51,"column":114},"end":{"row":51,"column":115},"action":"insert","lines":["y"],"id":626}],[{"start":{"row":51,"column":115},"end":{"row":51,"column":116},"action":"insert","lines":[" "],"id":627}],[{"start":{"row":51,"column":116},"end":{"row":51,"column":117},"action":"insert","lines":["u"],"id":628}],[{"start":{"row":51,"column":117},"end":{"row":51,"column":118},"action":"insert","lines":["s"],"id":629}],[{"start":{"row":51,"column":118},"end":{"row":51,"column":119},"action":"insert","lines":["e"],"id":630}],[{"start":{"row":51,"column":119},"end":{"row":51,"column":120},"action":"insert","lines":["d"],"id":631}],[{"start":{"row":51,"column":120},"end":{"row":51,"column":121},"action":"insert","lines":[" "],"id":632}],[{"start":{"row":51,"column":102},"end":{"row":51,"column":106},"action":"remove","lines":["test"],"id":633},{"start":{"row":51,"column":102},"end":{"row":51,"column":103},"action":"insert","lines":["f"]}],[{"start":{"row":51,"column":103},"end":{"row":51,"column":104},"action":"insert","lines":["u"],"id":634}],[{"start":{"row":51,"column":104},"end":{"row":51,"column":105},"action":"insert","lines":["n"],"id":635}],[{"start":{"row":51,"column":105},"end":{"row":51,"column":106},"action":"insert","lines":["c"],"id":636}],[{"start":{"row":51,"column":106},"end":{"row":51,"column":107},"action":"insert","lines":["t"],"id":637}],[{"start":{"row":51,"column":107},"end":{"row":51,"column":108},"action":"insert","lines":["i"],"id":638}],[{"start":{"row":51,"column":108},"end":{"row":51,"column":109},"action":"insert","lines":["o"],"id":639}],[{"start":{"row":51,"column":109},"end":{"row":51,"column":110},"action":"insert","lines":["n"],"id":640}],[{"start":{"row":51,"column":125},"end":{"row":51,"column":143},"action":"remove","lines":[" is often used for"],"id":641},{"start":{"row":51,"column":125},"end":{"row":51,"column":126},"action":"insert","lines":["t"]}],[{"start":{"row":51,"column":126},"end":{"row":51,"column":127},"action":"insert","lines":["o"],"id":642}],[{"start":{"row":51,"column":127},"end":{"row":51,"column":128},"action":"insert","lines":[" "],"id":643}],[{"start":{"row":51,"column":128},"end":{"row":51,"column":129},"action":"insert","lines":["t"],"id":644}],[{"start":{"row":51,"column":129},"end":{"row":51,"column":130},"action":"insert","lines":["e"],"id":645}],[{"start":{"row":51,"column":130},"end":{"row":51,"column":131},"action":"insert","lines":["s"],"id":646}],[{"start":{"row":51,"column":131},"end":{"row":51,"column":132},"action":"insert","lines":["t"],"id":647}],[{"start":{"row":51,"column":125},"end":{"row":51,"column":127},"action":"remove","lines":["to"],"id":648},{"start":{"row":51,"column":125},"end":{"row":51,"column":126},"action":"insert","lines":["f"]}],[{"start":{"row":51,"column":126},"end":{"row":51,"column":127},"action":"insert","lines":["o"],"id":649}],[{"start":{"row":51,"column":127},"end":{"row":51,"column":128},"action":"insert","lines":["r"],"id":650}],[{"start":{"row":51,"column":133},"end":{"row":51,"column":134},"action":"insert","lines":["i"],"id":651}],[{"start":{"row":51,"column":134},"end":{"row":51,"column":135},"action":"insert","lines":["n"],"id":652}],[{"start":{"row":51,"column":135},"end":{"row":51,"column":136},"action":"insert","lines":["g"],"id":653}],[{"start":{"row":52,"column":52},"end":{"row":52,"column":53},"action":"insert","lines":[","],"id":654}],[{"start":{"row":63,"column":0},"end":{"row":63,"column":15},"action":"remove","lines":["The benchmarks "],"id":655},{"start":{"row":63,"column":0},"end":{"row":63,"column":1},"action":"insert","lines":["W"]}],[{"start":{"row":63,"column":1},"end":{"row":63,"column":2},"action":"insert","lines":["e"],"id":656}],[{"start":{"row":63,"column":2},"end":{"row":63,"column":3},"action":"insert","lines":[" "],"id":657}],[{"start":{"row":63,"column":3},"end":{"row":63,"column":4},"action":"insert","lines":["w"],"id":658}],[{"start":{"row":63,"column":4},"end":{"row":63,"column":5},"action":"insert","lines":["i"],"id":659}],[{"start":{"row":63,"column":5},"end":{"row":63,"column":6},"action":"insert","lines":["l"],"id":660}],[{"start":{"row":63,"column":6},"end":{"row":63,"column":7},"action":"insert","lines":["l"],"id":661}],[{"start":{"row":63,"column":7},"end":{"row":63,"column":8},"action":"insert","lines":[" "],"id":662}],[{"start":{"row":63,"column":8},"end":{"row":63,"column":9},"action":"insert","lines":["g"],"id":663}],[{"start":{"row":63,"column":9},"end":{"row":63,"column":10},"action":"insert","lines":["e"],"id":664}],[{"start":{"row":63,"column":10},"end":{"row":63,"column":11},"action":"insert","lines":["n"],"id":665}],[{"start":{"row":63,"column":11},"end":{"row":63,"column":12},"action":"insert","lines":["e"],"id":666}],[{"start":{"row":63,"column":12},"end":{"row":63,"column":13},"action":"insert","lines":["r"],"id":667}],[{"start":{"row":63,"column":13},"end":{"row":63,"column":14},"action":"insert","lines":["a"],"id":668}],[{"start":{"row":63,"column":14},"end":{"row":63,"column":15},"action":"insert","lines":["t"],"id":669}],[{"start":{"row":63,"column":15},"end":{"row":63,"column":16},"action":"insert","lines":["e"],"id":670}],[{"start":{"row":63,"column":16},"end":{"row":63,"column":17},"action":"insert","lines":[" "],"id":671}],[{"start":{"row":63,"column":3},"end":{"row":63,"column":7},"action":"remove","lines":["will"],"id":672}],[{"start":{"row":63,"column":2},"end":{"row":63,"column":3},"action":"remove","lines":[" "],"id":673}],[{"start":{"row":63,"column":0},"end":{"row":63,"column":12},"action":"remove","lines":["We generate "],"id":674},{"start":{"row":63,"column":0},"end":{"row":63,"column":1},"action":"insert","lines":["F"]}],[{"start":{"row":63,"column":1},"end":{"row":63,"column":2},"action":"insert","lines":["o"],"id":675}],[{"start":{"row":63,"column":2},"end":{"row":63,"column":3},"action":"insert","lines":["r"],"id":676}],[{"start":{"row":63,"column":3},"end":{"row":63,"column":4},"action":"insert","lines":[" "],"id":677}],[{"start":{"row":63,"column":4},"end":{"row":63,"column":5},"action":"insert","lines":["t"],"id":678}],[{"start":{"row":63,"column":5},"end":{"row":63,"column":6},"action":"insert","lines":["h"],"id":679}],[{"start":{"row":63,"column":6},"end":{"row":63,"column":7},"action":"insert","lines":["e"],"id":680}],[{"start":{"row":63,"column":7},"end":{"row":63,"column":8},"action":"insert","lines":[" "],"id":681}],[{"start":{"row":63,"column":8},"end":{"row":63,"column":9},"action":"insert","lines":["b"],"id":682}],[{"start":{"row":63,"column":9},"end":{"row":63,"column":10},"action":"insert","lines":["e"],"id":683}],[{"start":{"row":63,"column":10},"end":{"row":63,"column":11},"action":"insert","lines":["n"],"id":684}],[{"start":{"row":63,"column":11},"end":{"row":63,"column":12},"action":"insert","lines":["c"],"id":685}],[{"start":{"row":63,"column":12},"end":{"row":63,"column":13},"action":"insert","lines":["h"],"id":686}],[{"start":{"row":63,"column":13},"end":{"row":63,"column":14},"action":"insert","lines":["m"],"id":687}],[{"start":{"row":63,"column":14},"end":{"row":63,"column":15},"action":"insert","lines":["a"],"id":688}],[{"start":{"row":63,"column":15},"end":{"row":63,"column":16},"action":"insert","lines":["r"],"id":689}],[{"start":{"row":63,"column":16},"end":{"row":63,"column":17},"action":"insert","lines":["k"],"id":690}],[{"start":{"row":63,"column":17},"end":{"row":63,"column":18},"action":"insert","lines":[" "],"id":691}],[{"start":{"row":63,"column":17},"end":{"row":63,"column":18},"action":"remove","lines":[" "],"id":692}],[{"start":{"row":63,"column":17},"end":{"row":63,"column":18},"action":"insert","lines":["s"],"id":693}],[{"start":{"row":63,"column":18},"end":{"row":63,"column":19},"action":"insert","lines":[" "],"id":694}],[{"start":{"row":63,"column":19},"end":{"row":63,"column":20},"action":"insert","lines":["w"],"id":695}],[{"start":{"row":63,"column":20},"end":{"row":63,"column":21},"action":"insert","lines":["e"],"id":696}],[{"start":{"row":63,"column":21},"end":{"row":63,"column":22},"action":"insert","lines":[" "],"id":697}],[{"start":{"row":63,"column":22},"end":{"row":63,"column":23},"action":"insert","lines":["g"],"id":698}],[{"start":{"row":63,"column":23},"end":{"row":63,"column":24},"action":"insert","lines":["e"],"id":699}],[{"start":{"row":63,"column":24},"end":{"row":63,"column":25},"action":"insert","lines":["n"],"id":700}],[{"start":{"row":63,"column":25},"end":{"row":63,"column":26},"action":"insert","lines":["e"],"id":701}],[{"start":{"row":63,"column":26},"end":{"row":63,"column":27},"action":"insert","lines":["r"],"id":702}],[{"start":{"row":63,"column":27},"end":{"row":63,"column":28},"action":"insert","lines":["a"],"id":703}],[{"start":{"row":63,"column":28},"end":{"row":63,"column":29},"action":"insert","lines":["t"],"id":704}],[{"start":{"row":63,"column":29},"end":{"row":63,"column":30},"action":"insert","lines":["e"],"id":705}],[{"start":{"row":63,"column":30},"end":{"row":63,"column":31},"action":"insert","lines":[" "],"id":706}],[{"start":{"row":63,"column":31},"end":{"row":63,"column":32},"action":"insert","lines":["v"],"id":707}],[{"start":{"row":63,"column":32},"end":{"row":63,"column":33},"action":"insert","lines":["a"],"id":708}],[{"start":{"row":63,"column":33},"end":{"row":63,"column":34},"action":"insert","lines":["r"],"id":709}],[{"start":{"row":63,"column":34},"end":{"row":63,"column":35},"action":"insert","lines":["y"],"id":710}],[{"start":{"row":63,"column":35},"end":{"row":63,"column":36},"action":"insert","lines":["i"],"id":711}],[{"start":{"row":63,"column":36},"end":{"row":63,"column":37},"action":"insert","lines":["n"],"id":712}],[{"start":{"row":63,"column":37},"end":{"row":63,"column":38},"action":"insert","lines":["g"],"id":713}],[{"start":{"row":63,"column":38},"end":{"row":63,"column":39},"action":"insert","lines":[" "],"id":714}],[{"start":{"row":63,"column":39},"end":{"row":63,"column":40},"action":"insert","lines":["s"],"id":715}],[{"start":{"row":63,"column":40},"end":{"row":63,"column":41},"action":"insert","lines":["e"],"id":716}],[{"start":{"row":63,"column":41},"end":{"row":63,"column":42},"action":"insert","lines":["t"],"id":717}],[{"start":{"row":63,"column":42},"end":{"row":63,"column":43},"action":"insert","lines":["s"],"id":718}],[{"start":{"row":63,"column":43},"end":{"row":63,"column":44},"action":"insert","lines":[" "],"id":719}],[{"start":{"row":63,"column":44},"end":{"row":63,"column":45},"action":"insert","lines":["o"],"id":720}],[{"start":{"row":63,"column":45},"end":{"row":63,"column":46},"action":"insert","lines":["o"],"id":721}],[{"start":{"row":63,"column":46},"end":{"row":63,"column":47},"action":"insert","lines":["f"],"id":722}],[{"start":{"row":63,"column":47},"end":{"row":63,"column":48},"action":"insert","lines":[" "],"id":723}],[{"start":{"row":63,"column":48},"end":{"row":63,"column":49},"action":"insert","lines":["a"],"id":724}],[{"start":{"row":63,"column":49},"end":{"row":63,"column":50},"action":"insert","lines":["r"],"id":725}],[{"start":{"row":63,"column":50},"end":{"row":63,"column":51},"action":"insert","lines":["g"],"id":726}],[{"start":{"row":63,"column":51},"end":{"row":63,"column":52},"action":"insert","lines":["u"],"id":727}],[{"start":{"row":63,"column":52},"end":{"row":63,"column":53},"action":"insert","lines":["m"],"id":728}],[{"start":{"row":63,"column":53},"end":{"row":63,"column":54},"action":"insert","lines":["e"],"id":729}],[{"start":{"row":63,"column":54},"end":{"row":63,"column":55},"action":"insert","lines":["n"],"id":730}],[{"start":{"row":63,"column":55},"end":{"row":63,"column":56},"action":"insert","lines":["t"],"id":731}],[{"start":{"row":63,"column":56},"end":{"row":63,"column":57},"action":"insert","lines":["s"],"id":732}],[{"start":{"row":63,"column":57},"end":{"row":63,"column":58},"action":"insert","lines":[","],"id":733}],[{"start":{"row":63,"column":58},"end":{"row":63,"column":59},"action":"insert","lines":[" "],"id":734}],[{"start":{"row":63,"column":59},"end":{"row":63,"column":60},"action":"insert","lines":["o"],"id":735}],[{"start":{"row":63,"column":60},"end":{"row":63,"column":61},"action":"insert","lines":["f"],"id":736}],[{"start":{"row":63,"column":61},"end":{"row":63,"column":62},"action":"insert","lines":[" "],"id":737}],[{"start":{"row":63,"column":62},"end":{"row":63,"column":63},"action":"insert","lines":["v"],"id":738}],[{"start":{"row":63,"column":63},"end":{"row":63,"column":64},"action":"insert","lines":["a"],"id":739}],[{"start":{"row":63,"column":64},"end":{"row":63,"column":65},"action":"insert","lines":["r"],"id":740}],[{"start":{"row":63,"column":65},"end":{"row":63,"column":66},"action":"insert","lines":["y"],"id":741}],[{"start":{"row":63,"column":66},"end":{"row":63,"column":67},"action":"insert","lines":["i"],"id":742}],[{"start":{"row":63,"column":67},"end":{"row":63,"column":68},"action":"insert","lines":["n"],"id":743}],[{"start":{"row":63,"column":68},"end":{"row":63,"column":69},"action":"insert","lines":["g"],"id":744}],[{"start":{"row":63,"column":69},"end":{"row":63,"column":70},"action":"insert","lines":[" "],"id":745}],[{"start":{"row":63,"column":39},"end":{"row":63,"column":43},"action":"remove","lines":["sets"],"id":746},{"start":{"row":63,"column":39},"end":{"row":63,"column":40},"action":"insert","lines":["n"]}],[{"start":{"row":63,"column":40},"end":{"row":63,"column":41},"action":"insert","lines":["u"],"id":747}],[{"start":{"row":63,"column":41},"end":{"row":63,"column":42},"action":"insert","lines":["m"],"id":748}],[{"start":{"row":63,"column":42},"end":{"row":63,"column":43},"action":"insert","lines":["b"],"id":749}],[{"start":{"row":63,"column":43},"end":{"row":63,"column":44},"action":"insert","lines":["e"],"id":750}],[{"start":{"row":63,"column":44},"end":{"row":63,"column":45},"action":"insert","lines":["r"],"id":751}],[{"start":{"row":63,"column":45},"end":{"row":63,"column":46},"action":"insert","lines":["s"],"id":752}],[{"start":{"row":63,"column":47},"end":{"row":63,"column":48},"action":"remove","lines":["o"],"id":753}],[{"start":{"row":63,"column":50},"end":{"row":63,"column":51},"action":"insert","lines":["c"],"id":754}],[{"start":{"row":63,"column":51},"end":{"row":63,"column":52},"action":"insert","lines":["o"],"id":755}],[{"start":{"row":63,"column":52},"end":{"row":63,"column":53},"action":"insert","lines":["m"],"id":756}],[{"start":{"row":63,"column":53},"end":{"row":63,"column":54},"action":"insert","lines":["p"],"id":757}],[{"start":{"row":63,"column":54},"end":{"row":63,"column":55},"action":"insert","lines":["l"],"id":758}],[{"start":{"row":63,"column":55},"end":{"row":63,"column":56},"action":"insert","lines":["e"],"id":759}],[{"start":{"row":63,"column":56},"end":{"row":63,"column":57},"action":"insert","lines":["x"],"id":760}],[{"start":{"row":63,"column":57},"end":{"row":63,"column":58},"action":"insert","lines":[" "],"id":761}],[{"start":{"row":63,"column":67},"end":{"row":63,"column":68},"action":"insert","lines":[" "],"id":762}],[{"start":{"row":63,"column":68},"end":{"row":63,"column":69},"action":"insert","lines":["("],"id":763}],[{"start":{"row":63,"column":69},"end":{"row":63,"column":70},"action":"insert","lines":["1"],"id":764}],[{"start":{"row":63,"column":70},"end":{"row":63,"column":71},"action":"insert","lines":[" "],"id":765}],[{"start":{"row":63,"column":71},"end":{"row":63,"column":72},"action":"insert","lines":["t"],"id":766}],[{"start":{"row":63,"column":72},"end":{"row":63,"column":73},"action":"insert","lines":["o"],"id":767}],[{"start":{"row":63,"column":73},"end":{"row":63,"column":74},"action":"insert","lines":[" "],"id":768}],[{"start":{"row":63,"column":74},"end":{"row":63,"column":75},"action":"insert","lines":["8"],"id":769}],[{"start":{"row":63,"column":75},"end":{"row":63,"column":76},"action":"insert","lines":[" "],"id":770}],[{"start":{"row":63,"column":76},"end":{"row":63,"column":77},"action":"insert","lines":["a"],"id":771}],[{"start":{"row":63,"column":77},"end":{"row":63,"column":78},"action":"insert","lines":["r"],"id":772}],[{"start":{"row":63,"column":78},"end":{"row":63,"column":79},"action":"insert","lines":["g"],"id":773}],[{"start":{"row":63,"column":79},"end":{"row":63,"column":80},"action":"insert","lines":["y"],"id":774}],[{"start":{"row":63,"column":80},"end":{"row":63,"column":81},"action":"insert","lines":["="],"id":775}],[{"start":{"row":63,"column":80},"end":{"row":63,"column":81},"action":"remove","lines":["="],"id":776}],[{"start":{"row":63,"column":79},"end":{"row":63,"column":80},"action":"remove","lines":["y"],"id":777}],[{"start":{"row":63,"column":79},"end":{"row":63,"column":80},"action":"insert","lines":["u"],"id":778}],[{"start":{"row":63,"column":80},"end":{"row":63,"column":81},"action":"insert","lines":["m"],"id":779}],[{"start":{"row":63,"column":81},"end":{"row":63,"column":82},"action":"insert","lines":["e"],"id":780}],[{"start":{"row":63,"column":82},"end":{"row":63,"column":83},"action":"insert","lines":["n"],"id":781}],[{"start":{"row":63,"column":83},"end":{"row":63,"column":84},"action":"insert","lines":["t"],"id":782}],[{"start":{"row":63,"column":84},"end":{"row":63,"column":85},"action":"insert","lines":["s"],"id":783}],[{"start":{"row":63,"column":85},"end":{"row":63,"column":86},"action":"insert","lines":[")"],"id":784}],[{"start":{"row":63,"column":88},"end":{"row":63,"column":90},"action":"remove","lines":["of"],"id":785},{"start":{"row":63,"column":88},"end":{"row":63,"column":89},"action":"insert","lines":["w"]}],[{"start":{"row":63,"column":89},"end":{"row":63,"column":90},"action":"insert","lines":["i"],"id":786}],[{"start":{"row":63,"column":90},"end":{"row":63,"column":91},"action":"insert","lines":["t"],"id":787}],[{"start":{"row":63,"column":91},"end":{"row":63,"column":92},"action":"insert","lines":["h"],"id":788}],[{"start":{"row":63,"column":101},"end":{"row":63,"column":102},"action":"insert","lines":["d"],"id":789}],[{"start":{"row":63,"column":102},"end":{"row":63,"column":103},"action":"insert","lines":["e"],"id":790}],[{"start":{"row":63,"column":103},"end":{"row":63,"column":104},"action":"insert","lines":["g"],"id":791}],[{"start":{"row":63,"column":104},"end":{"row":63,"column":105},"action":"insert","lines":["r"],"id":792}],[{"start":{"row":63,"column":105},"end":{"row":63,"column":106},"action":"insert","lines":["e"],"id":793}],[{"start":{"row":63,"column":106},"end":{"row":63,"column":107},"action":"insert","lines":["e"],"id":794}],[{"start":{"row":63,"column":107},"end":{"row":63,"column":108},"action":"insert","lines":["s"],"id":795}],[{"start":{"row":63,"column":108},"end":{"row":63,"column":109},"action":"insert","lines":[" "],"id":796}],[{"start":{"row":63,"column":109},"end":{"row":63,"column":110},"action":"insert","lines":["o"],"id":797}],[{"start":{"row":63,"column":110},"end":{"row":63,"column":111},"action":"insert","lines":["f"],"id":798}],[{"start":{"row":63,"column":111},"end":{"row":63,"column":112},"action":"insert","lines":[" "],"id":799}],[{"start":{"row":63,"column":112},"end":{"row":63,"column":113},"action":"insert","lines":["v"],"id":800}],[{"start":{"row":63,"column":113},"end":{"row":63,"column":114},"action":"insert","lines":["a"],"id":801}],[{"start":{"row":63,"column":114},"end":{"row":63,"column":115},"action":"insert","lines":["r"],"id":802}],[{"start":{"row":63,"column":115},"end":{"row":63,"column":116},"action":"insert","lines":["i"],"id":803}],[{"start":{"row":63,"column":116},"end":{"row":63,"column":117},"action":"insert","lines":["a"],"id":804}],[{"start":{"row":63,"column":117},"end":{"row":63,"column":118},"action":"insert","lines":["n"],"id":805}],[{"start":{"row":63,"column":118},"end":{"row":63,"column":119},"action":"insert","lines":["c"],"id":806}],[{"start":{"row":63,"column":119},"end":{"row":63,"column":120},"action":"insert","lines":["e"],"id":807}],[{"start":{"row":63,"column":120},"end":{"row":63,"column":121},"action":"insert","lines":[","],"id":808}],[{"start":{"row":63,"column":121},"end":{"row":63,"column":122},"action":"insert","lines":[" "],"id":809}],[{"start":{"row":63,"column":122},"end":{"row":63,"column":123},"action":"insert","lines":["t"],"id":810}],[{"start":{"row":63,"column":123},"end":{"row":63,"column":124},"action":"insert","lines":["o"],"id":811}],[{"start":{"row":63,"column":124},"end":{"row":63,"column":125},"action":"insert","lines":[" "],"id":812}],[{"start":{"row":63,"column":125},"end":{"row":63,"column":126},"action":"insert","lines":["i"],"id":813}],[{"start":{"row":63,"column":126},"end":{"row":63,"column":127},"action":"insert","lines":["n"],"id":814}],[{"start":{"row":63,"column":127},"end":{"row":63,"column":128},"action":"insert","lines":["c"],"id":815}],[{"start":{"row":63,"column":128},"end":{"row":63,"column":129},"action":"insert","lines":["r"],"id":816}],[{"start":{"row":63,"column":129},"end":{"row":63,"column":130},"action":"insert","lines":["e"],"id":817}],[{"start":{"row":63,"column":130},"end":{"row":63,"column":131},"action":"insert","lines":["a"],"id":818}],[{"start":{"row":63,"column":131},"end":{"row":63,"column":132},"action":"insert","lines":["s"],"id":819}],[{"start":{"row":63,"column":132},"end":{"row":63,"column":133},"action":"insert","lines":["e"],"id":820}],[{"start":{"row":63,"column":133},"end":{"row":63,"column":134},"action":"insert","lines":[" "],"id":821}],[{"start":{"row":63,"column":134},"end":{"row":63,"column":135},"action":"insert","lines":["a"],"id":822}],[{"start":{"row":63,"column":135},"end":{"row":63,"column":136},"action":"insert","lines":["n"],"id":823}],[{"start":{"row":63,"column":136},"end":{"row":63,"column":137},"action":"insert","lines":["d"],"id":824}],[{"start":{"row":63,"column":137},"end":{"row":63,"column":138},"action":"insert","lines":[" "],"id":825}],[{"start":{"row":63,"column":138},"end":{"row":63,"column":139},"action":"insert","lines":["d"],"id":826}],[{"start":{"row":63,"column":139},"end":{"row":63,"column":140},"action":"insert","lines":["e"],"id":827}],[{"start":{"row":63,"column":140},"end":{"row":63,"column":141},"action":"insert","lines":["c"],"id":828}],[{"start":{"row":63,"column":141},"end":{"row":63,"column":142},"action":"insert","lines":["r"],"id":829}],[{"start":{"row":63,"column":142},"end":{"row":63,"column":143},"action":"insert","lines":["e"],"id":830}],[{"start":{"row":63,"column":143},"end":{"row":63,"column":144},"action":"insert","lines":["a"],"id":831}],[{"start":{"row":63,"column":144},"end":{"row":63,"column":145},"action":"insert","lines":["s"],"id":832}],[{"start":{"row":63,"column":145},"end":{"row":63,"column":146},"action":"insert","lines":["e"],"id":833}],[{"start":{"row":63,"column":146},"end":{"row":63,"column":147},"action":"insert","lines":[" "],"id":834}],[{"start":{"row":63,"column":147},"end":{"row":63,"column":148},"action":"insert","lines":["c"],"id":835}],[{"start":{"row":63,"column":148},"end":{"row":63,"column":149},"action":"insert","lines":["a"],"id":836}],[{"start":{"row":63,"column":149},"end":{"row":63,"column":150},"action":"insert","lines":["c"],"id":837}],[{"start":{"row":63,"column":150},"end":{"row":63,"column":151},"action":"insert","lines":["h"],"id":838}],[{"start":{"row":63,"column":151},"end":{"row":63,"column":152},"action":"insert","lines":["e"],"id":839}],[{"start":{"row":63,"column":152},"end":{"row":63,"column":153},"action":"insert","lines":[" "],"id":840}],[{"start":{"row":63,"column":153},"end":{"row":63,"column":154},"action":"insert","lines":["h"],"id":841}],[{"start":{"row":63,"column":154},"end":{"row":63,"column":155},"action":"insert","lines":["i"],"id":842}],[{"start":{"row":63,"column":155},"end":{"row":63,"column":156},"action":"insert","lines":["t"],"id":843}],[{"start":{"row":63,"column":156},"end":{"row":63,"column":157},"action":"insert","lines":["s"],"id":844}],[{"start":{"row":63,"column":157},"end":{"row":63,"column":158},"action":"insert","lines":[" "],"id":845}],[{"start":{"row":63,"column":158},"end":{"row":63,"column":159},"action":"insert","lines":["a"],"id":846}],[{"start":{"row":63,"column":159},"end":{"row":63,"column":160},"action":"insert","lines":["n"],"id":847}],[{"start":{"row":63,"column":160},"end":{"row":63,"column":161},"action":"insert","lines":["d"],"id":848}],[{"start":{"row":63,"column":161},"end":{"row":63,"column":162},"action":"insert","lines":[" "],"id":849}],[{"start":{"row":63,"column":162},"end":{"row":63,"column":163},"action":"insert","lines":["m"],"id":850}],[{"start":{"row":63,"column":163},"end":{"row":63,"column":164},"action":"insert","lines":["i"],"id":851}],[{"start":{"row":63,"column":164},"end":{"row":63,"column":165},"action":"insert","lines":["s"],"id":852}],[{"start":{"row":63,"column":165},"end":{"row":63,"column":166},"action":"insert","lines":["s"],"id":853}],[{"start":{"row":63,"column":166},"end":{"row":63,"column":167},"action":"insert","lines":["e"],"id":854}],[{"start":{"row":63,"column":167},"end":{"row":63,"column":168},"action":"insert","lines":["s"],"id":855}],[{"start":{"row":63,"column":168},"end":{"row":63,"column":169},"action":"insert","lines":["."],"id":856}],[{"start":{"row":63,"column":169},"end":{"row":64,"column":0},"action":"insert","lines":["",""],"id":857}],[{"start":{"row":64,"column":0},"end":{"row":64,"column":1},"action":"insert","lines":["T"],"id":858}],[{"start":{"row":64,"column":1},"end":{"row":64,"column":2},"action":"insert","lines":["o"],"id":859}],[{"start":{"row":64,"column":2},"end":{"row":64,"column":3},"action":"insert","lines":[" "],"id":860}],[{"start":{"row":64,"column":2},"end":{"row":64,"column":3},"action":"remove","lines":[" "],"id":861}],[{"start":{"row":64,"column":1},"end":{"row":64,"column":2},"action":"remove","lines":["o"],"id":862}],[{"start":{"row":64,"column":0},"end":{"row":64,"column":1},"action":"remove","lines":["T"],"id":863}],[{"start":{"row":64,"column":0},"end":{"row":64,"column":1},"action":"insert","lines":["F"],"id":864}],[{"start":{"row":64,"column":1},"end":{"row":64,"column":2},"action":"insert","lines":["o"],"id":865}],[{"start":{"row":64,"column":2},"end":{"row":64,"column":3},"action":"insert","lines":["r"],"id":866}],[{"start":{"row":64,"column":3},"end":{"row":64,"column":4},"action":"insert","lines":[" "],"id":867}],[{"start":{"row":64,"column":4},"end":{"row":64,"column":5},"action":"insert","lines":["f"],"id":868}],[{"start":{"row":64,"column":5},"end":{"row":64,"column":6},"action":"insert","lines":["u"],"id":869}],[{"start":{"row":64,"column":6},"end":{"row":64,"column":7},"action":"insert","lines":["l"],"id":870}],[{"start":{"row":64,"column":7},"end":{"row":64,"column":8},"action":"insert","lines":["l"],"id":871}],[{"start":{"row":64,"column":8},"end":{"row":64,"column":9},"action":"insert","lines":[" "],"id":872}],[{"start":{"row":64,"column":9},"end":{"row":64,"column":10},"action":"insert","lines":["d"],"id":873}],[{"start":{"row":64,"column":10},"end":{"row":64,"column":11},"action":"insert","lines":["e"],"id":874}],[{"start":{"row":64,"column":11},"end":{"row":64,"column":12},"action":"insert","lines":["t"],"id":875}],[{"start":{"row":64,"column":12},"end":{"row":64,"column":13},"action":"insert","lines":["a"],"id":876}],[{"start":{"row":64,"column":13},"end":{"row":64,"column":14},"action":"insert","lines":["i"],"id":877}],[{"start":{"row":64,"column":14},"end":{"row":64,"column":15},"action":"insert","lines":["l"],"id":878}],[{"start":{"row":64,"column":15},"end":{"row":64,"column":16},"action":"insert","lines":["s"],"id":879}],[{"start":{"row":64,"column":16},"end":{"row":64,"column":17},"action":"insert","lines":[" "],"id":880}],[{"start":{"row":64,"column":17},"end":{"row":64,"column":18},"action":"insert","lines":["s"],"id":881}],[{"start":{"row":64,"column":18},"end":{"row":64,"column":19},"action":"insert","lines":["e"],"id":882}],[{"start":{"row":64,"column":19},"end":{"row":64,"column":20},"action":"insert","lines":["e"],"id":883}],[{"start":{"row":64,"column":20},"end":{"row":64,"column":21},"action":"insert","lines":[" "],"id":884}],[{"start":{"row":64,"column":21},"end":{"row":64,"column":22},"action":"insert","lines":["t"],"id":885}],[{"start":{"row":64,"column":22},"end":{"row":64,"column":23},"action":"insert","lines":["h"],"id":886}],[{"start":{"row":64,"column":23},"end":{"row":64,"column":24},"action":"insert","lines":["e"],"id":887}],[{"start":{"row":64,"column":24},"end":{"row":64,"column":25},"action":"insert","lines":[" "],"id":888}],[{"start":{"row":64,"column":25},"end":{"row":64,"column":26},"action":"insert","lines":["b"],"id":889}],[{"start":{"row":64,"column":26},"end":{"row":64,"column":27},"action":"insert","lines":["e"],"id":890}],[{"start":{"row":64,"column":27},"end":{"row":64,"column":28},"action":"insert","lines":["n"],"id":891}],[{"start":{"row":64,"column":28},"end":{"row":64,"column":29},"action":"insert","lines":["c"],"id":892}],[{"start":{"row":64,"column":29},"end":{"row":64,"column":30},"action":"insert","lines":["h"],"id":893}],[{"start":{"row":64,"column":30},"end":{"row":64,"column":31},"action":"insert","lines":["a"],"id":894}],[{"start":{"row":64,"column":31},"end":{"row":64,"column":32},"action":"insert","lines":["m"],"id":895}],[{"start":{"row":64,"column":32},"end":{"row":64,"column":33},"action":"insert","lines":["r"],"id":896}],[{"start":{"row":64,"column":33},"end":{"row":64,"column":34},"action":"insert","lines":["k"],"id":897}],[{"start":{"row":64,"column":34},"end":{"row":64,"column":35},"action":"insert","lines":["s"],"id":898}],[{"start":{"row":64,"column":34},"end":{"row":64,"column":35},"action":"remove","lines":["s"],"id":899}],[{"start":{"row":64,"column":33},"end":{"row":64,"column":34},"action":"remove","lines":["k"],"id":900}],[{"start":{"row":64,"column":32},"end":{"row":64,"column":33},"action":"remove","lines":["r"],"id":901}],[{"start":{"row":64,"column":32},"end":{"row":64,"column":33},"action":"insert","lines":["a"],"id":902}],[{"start":{"row":64,"column":33},"end":{"row":64,"column":34},"action":"insert","lines":["r"],"id":903}],[{"start":{"row":64,"column":34},"end":{"row":64,"column":35},"action":"insert","lines":["k"],"id":904}],[{"start":{"row":64,"column":35},"end":{"row":64,"column":36},"action":"insert","lines":["s"],"id":905}],[{"start":{"row":64,"column":25},"end":{"row":64,"column":26},"action":"insert","lines":["f"],"id":906}],[{"start":{"row":64,"column":26},"end":{"row":64,"column":27},"action":"insert","lines":["u"],"id":907}],[{"start":{"row":64,"column":27},"end":{"row":64,"column":28},"action":"insert","lines":["l"],"id":908}],[{"start":{"row":64,"column":28},"end":{"row":64,"column":29},"action":"insert","lines":["l"],"id":909}],[{"start":{"row":64,"column":29},"end":{"row":64,"column":30},"action":"insert","lines":[" "],"id":910}],[{"start":{"row":64,"column":25},"end":{"row":64,"column":30},"action":"remove","lines":["full "],"id":911}],[{"start":{"row":64,"column":35},"end":{"row":64,"column":36},"action":"remove","lines":["s"],"id":912}],[{"start":{"row":64,"column":35},"end":{"row":64,"column":36},"action":"insert","lines":[" "],"id":913}],[{"start":{"row":64,"column":36},"end":{"row":64,"column":37},"action":"insert","lines":["s"],"id":914}],[{"start":{"row":64,"column":37},"end":{"row":64,"column":38},"action":"insert","lines":["o"],"id":915}],[{"start":{"row":64,"column":38},"end":{"row":64,"column":39},"action":"insert","lines":["u"],"id":916}],[{"start":{"row":64,"column":39},"end":{"row":64,"column":40},"action":"insert","lines":["c"],"id":917}],[{"start":{"row":64,"column":39},"end":{"row":64,"column":40},"action":"remove","lines":["c"],"id":918}],[{"start":{"row":64,"column":39},"end":{"row":64,"column":40},"action":"insert","lines":["r"],"id":919}],[{"start":{"row":64,"column":40},"end":{"row":64,"column":41},"action":"insert","lines":["c"],"id":920}],[{"start":{"row":64,"column":41},"end":{"row":64,"column":42},"action":"insert","lines":["e"],"id":921}],[{"start":{"row":64,"column":42},"end":{"row":64,"column":43},"action":"insert","lines":[" "],"id":922}],[{"start":{"row":64,"column":43},"end":{"row":64,"column":44},"action":"insert","lines":["a"],"id":923}],[{"start":{"row":64,"column":44},"end":{"row":64,"column":45},"action":"insert","lines":["t"],"id":924}],[{"start":{"row":64,"column":45},"end":{"row":64,"column":46},"action":"insert","lines":[" "],"id":925}],[{"start":{"row":64,"column":46},"end":{"row":64,"column":47},"action":"insert","lines":["t"],"id":926}],[{"start":{"row":64,"column":47},"end":{"row":64,"column":48},"action":"insert","lines":["h"],"id":927}],[{"start":{"row":64,"column":48},"end":{"row":64,"column":49},"action":"insert","lines":["i"],"id":928}],[{"start":{"row":64,"column":49},"end":{"row":64,"column":50},"action":"insert","lines":["s"],"id":929}],[{"start":{"row":64,"column":50},"end":{"row":64,"column":51},"action":"insert","lines":[" "],"id":930}],[{"start":{"row":64,"column":51},"end":{"row":64,"column":52},"action":"insert","lines":["p"],"id":931}],[{"start":{"row":64,"column":52},"end":{"row":64,"column":53},"action":"insert","lines":["r"],"id":932}],[{"start":{"row":64,"column":53},"end":{"row":64,"column":54},"action":"insert","lines":["o"],"id":933}],[{"start":{"row":64,"column":53},"end":{"row":64,"column":54},"action":"remove","lines":["o"],"id":934}],[{"start":{"row":64,"column":52},"end":{"row":64,"column":53},"action":"remove","lines":["r"],"id":935}],[{"start":{"row":64,"column":51},"end":{"row":64,"column":52},"action":"remove","lines":["p"],"id":936}],[{"start":{"row":64,"column":50},"end":{"row":64,"column":51},"action":"remove","lines":[" "],"id":937}],[{"start":{"row":64,"column":49},"end":{"row":64,"column":50},"action":"remove","lines":["s"],"id":938}],[{"start":{"row":64,"column":48},"end":{"row":64,"column":49},"action":"remove","lines":["i"],"id":939}],[{"start":{"row":64,"column":48},"end":{"row":64,"column":49},"action":"insert","lines":["e"],"id":940}],[{"start":{"row":64,"column":49},"end":{"row":64,"column":50},"action":"insert","lines":[" "],"id":941}],[{"start":{"row":64,"column":50},"end":{"row":64,"column":51},"action":"insert","lines":["m"],"id":942}],[{"start":{"row":64,"column":51},"end":{"row":64,"column":52},"action":"insert","lines":["e"],"id":943}],[{"start":{"row":64,"column":52},"end":{"row":64,"column":53},"action":"insert","lines":["m"],"id":944}],[{"start":{"row":64,"column":53},"end":{"row":64,"column":54},"action":"insert","lines":["o"],"id":945}],[{"start":{"row":64,"column":54},"end":{"row":64,"column":55},"action":"insert","lines":["i"],"id":946}],[{"start":{"row":64,"column":55},"end":{"row":64,"column":56},"action":"insert","lines":["z"],"id":947}],[{"start":{"row":64,"column":56},"end":{"row":64,"column":57},"action":"insert","lines":["e"],"id":948}],[{"start":{"row":64,"column":57},"end":{"row":64,"column":58},"action":"insert","lines":["r"],"id":949}],[{"start":{"row":64,"column":58},"end":{"row":64,"column":59},"action":"insert","lines":["i"],"id":950}],[{"start":{"row":64,"column":59},"end":{"row":64,"column":60},"action":"insert","lines":["f"],"id":951}],[{"start":{"row":64,"column":60},"end":{"row":64,"column":61},"action":"insert","lines":["i"],"id":952}],[{"start":{"row":64,"column":61},"end":{"row":64,"column":62},"action":"insert","lines":["c"],"id":953}],[{"start":{"row":64,"column":62},"end":{"row":64,"column":63},"action":"insert","lines":["-"],"id":954}],[{"start":{"row":64,"column":63},"end":{"row":64,"column":64},"action":"insert","lines":["b"],"id":955}],[{"start":{"row":64,"column":64},"end":{"row":64,"column":65},"action":"insert","lines":["e"],"id":956}],[{"start":{"row":64,"column":65},"end":{"row":64,"column":66},"action":"insert","lines":["n"],"id":957}],[{"start":{"row":64,"column":66},"end":{"row":64,"column":67},"action":"insert","lines":["c"],"id":958}],[{"start":{"row":64,"column":67},"end":{"row":64,"column":68},"action":"insert","lines":["h"],"id":959}],[{"start":{"row":64,"column":68},"end":{"row":64,"column":69},"action":"insert","lines":["m"],"id":960}],[{"start":{"row":64,"column":69},"end":{"row":64,"column":70},"action":"insert","lines":["a"],"id":961}],[{"start":{"row":64,"column":70},"end":{"row":64,"column":71},"action":"insert","lines":["r"],"id":962}],[{"start":{"row":64,"column":71},"end":{"row":64,"column":72},"action":"insert","lines":["k"],"id":963}],[{"start":{"row":64,"column":72},"end":{"row":64,"column":73},"action":"insert","lines":["s"],"id":964}],[{"start":{"row":64,"column":73},"end":{"row":64,"column":74},"action":"insert","lines":[" "],"id":965}],[{"start":{"row":64,"column":74},"end":{"row":64,"column":75},"action":"insert","lines":["p"],"id":966}],[{"start":{"row":64,"column":75},"end":{"row":64,"column":76},"action":"insert","lines":["r"],"id":967}],[{"start":{"row":64,"column":76},"end":{"row":64,"column":77},"action":"insert","lines":["o"],"id":968}],[{"start":{"row":64,"column":77},"end":{"row":64,"column":78},"action":"insert","lines":["j"],"id":969}],[{"start":{"row":64,"column":78},"end":{"row":64,"column":79},"action":"insert","lines":["e"],"id":970}],[{"start":{"row":64,"column":79},"end":{"row":64,"column":80},"action":"insert","lines":["c"],"id":971}],[{"start":{"row":64,"column":80},"end":{"row":64,"column":81},"action":"insert","lines":["t"],"id":972}],[{"start":{"row":64,"column":81},"end":{"row":64,"column":82},"action":"insert","lines":["."],"id":973}],[{"start":{"row":64,"column":57},"end":{"row":64,"column":62},"action":"remove","lines":["rific"],"id":974},{"start":{"row":64,"column":57},"end":{"row":64,"column":58},"action":"insert","lines":["-"]}],[{"start":{"row":64,"column":58},"end":{"row":64,"column":59},"action":"insert","lines":["j"],"id":975}],[{"start":{"row":64,"column":59},"end":{"row":64,"column":60},"action":"insert","lines":["s"],"id":976}],[{"start":{"row":64,"column":60},"end":{"row":64,"column":61},"action":"insert","lines":["-"],"id":977}],[{"start":{"row":64,"column":61},"end":{"row":64,"column":62},"action":"insert","lines":["l"],"id":978}],[{"start":{"row":64,"column":62},"end":{"row":64,"column":63},"action":"insert","lines":["i"],"id":979}],[{"start":{"row":64,"column":63},"end":{"row":64,"column":64},"action":"insert","lines":["b"],"id":980}],[{"start":{"row":64,"column":64},"end":{"row":64,"column":65},"action":"insert","lines":["s"],"id":981}],[{"start":{"row":64,"column":50},"end":{"row":64,"column":51},"action":"insert","lines":["["],"id":983}],[{"start":{"row":64,"column":77},"end":{"row":64,"column":78},"action":"insert","lines":["]"],"id":984}],[{"start":{"row":64,"column":78},"end":{"row":64,"column":79},"action":"insert","lines":["("],"id":985}],[{"start":{"row":64,"column":79},"end":{"row":64,"column":80},"action":"insert","lines":[")"],"id":986}],[{"start":{"row":64,"column":79},"end":{"row":64,"column":134},"action":"insert","lines":["https://github.com/thinkloop/memoize-js-libs-benchmarks"],"id":987}],[{"start":{"row":64,"column":17},"end":{"row":64,"column":18},"action":"insert","lines":["c"],"id":988}],[{"start":{"row":64,"column":18},"end":{"row":64,"column":19},"action":"insert","lines":["h"],"id":989}],[{"start":{"row":64,"column":19},"end":{"row":64,"column":20},"action":"insert","lines":["e"],"id":990}],[{"start":{"row":64,"column":20},"end":{"row":64,"column":21},"action":"insert","lines":["c"],"id":991}],[{"start":{"row":64,"column":21},"end":{"row":64,"column":22},"action":"insert","lines":["k"],"id":992}],[{"start":{"row":64,"column":22},"end":{"row":64,"column":23},"action":"insert","lines":[" "],"id":993}],[{"start":{"row":64,"column":23},"end":{"row":64,"column":24},"action":"insert","lines":["o"],"id":994}],[{"start":{"row":64,"column":24},"end":{"row":64,"column":25},"action":"insert","lines":["u"],"id":995}],[{"start":{"row":64,"column":25},"end":{"row":64,"column":26},"action":"insert","lines":["t"],"id":996}],[{"start":{"row":64,"column":26},"end":{"row":64,"column":27},"action":"insert","lines":[" "],"id":997}],[{"start":{"row":64,"column":27},"end":{"row":64,"column":28},"action":"insert","lines":["t"],"id":998}],[{"start":{"row":64,"column":28},"end":{"row":64,"column":29},"action":"insert","lines":["h"],"id":999}],[{"start":{"row":64,"column":29},"end":{"row":64,"column":30},"action":"insert","lines":["e"],"id":1000}],[{"start":{"row":64,"column":30},"end":{"row":64,"column":31},"action":"insert","lines":[" "],"id":1001}],[{"start":{"row":64,"column":31},"end":{"row":64,"column":32},"action":"insert","lines":["s"],"id":1002}],[{"start":{"row":64,"column":32},"end":{"row":64,"column":33},"action":"insert","lines":["o"],"id":1003}],[{"start":{"row":64,"column":33},"end":{"row":64,"column":34},"action":"insert","lines":["u"],"id":1004}],[{"start":{"row":64,"column":34},"end":{"row":64,"column":35},"action":"insert","lines":["r"],"id":1005}],[{"start":{"row":64,"column":35},"end":{"row":64,"column":36},"action":"insert","lines":["c"],"id":1006}],[{"start":{"row":64,"column":36},"end":{"row":64,"column":37},"action":"insert","lines":["e"],"id":1007}],[{"start":{"row":64,"column":37},"end":{"row":64,"column":38},"action":"insert","lines":[" "],"id":1008}],[{"start":{"row":64,"column":38},"end":{"row":64,"column":39},"action":"insert","lines":["a"],"id":1009}],[{"start":{"row":64,"column":39},"end":{"row":64,"column":40},"action":"insert","lines":["t"],"id":1010}],[{"start":{"row":64,"column":40},"end":{"row":64,"column":41},"action":"insert","lines":[" "],"id":1011}],[{"start":{"row":64,"column":41},"end":{"row":64,"column":70},"action":"remove","lines":["see the benchamark source at "],"id":1012}],[{"start":{"row":51,"column":102},"end":{"row":51,"column":110},"action":"remove","lines":["function"],"id":1013},{"start":{"row":51,"column":102},"end":{"row":51,"column":103},"action":"insert","lines":["t"]}],[{"start":{"row":51,"column":103},"end":{"row":51,"column":104},"action":"insert","lines":["e"],"id":1014}],[{"start":{"row":51,"column":104},"end":{"row":51,"column":105},"action":"insert","lines":["s"],"id":1015}],[{"start":{"row":51,"column":105},"end":{"row":51,"column":106},"action":"insert","lines":["t"],"id":1016}],[{"start":{"row":51,"column":125},"end":{"row":51,"column":132},"action":"remove","lines":["testing"],"id":1017}],[{"start":{"row":51,"column":124},"end":{"row":51,"column":125},"action":"remove","lines":[" "],"id":1018}],[{"start":{"row":63,"column":0},"end":{"row":63,"column":20},"action":"remove","lines":["For the benchmarks w"],"id":1019},{"start":{"row":63,"column":0},"end":{"row":63,"column":1},"action":"insert","lines":["W"]}],[{"start":{"row":63,"column":12},"end":{"row":63,"column":13},"action":"insert","lines":["a"],"id":1020}],[{"start":{"row":63,"column":13},"end":{"row":63,"column":14},"action":"insert","lines":[" "],"id":1021}],[{"start":{"row":63,"column":14},"end":{"row":63,"column":15},"action":"insert","lines":["l"],"id":1022}],[{"start":{"row":63,"column":15},"end":{"row":63,"column":16},"action":"insert","lines":["i"],"id":1023}],[{"start":{"row":63,"column":16},"end":{"row":63,"column":17},"action":"insert","lines":["s"],"id":1024}],[{"start":{"row":63,"column":17},"end":{"row":63,"column":18},"action":"insert","lines":["t"],"id":1025}],[{"start":{"row":63,"column":18},"end":{"row":63,"column":19},"action":"insert","lines":[" "],"id":1026}],[{"start":{"row":63,"column":19},"end":{"row":63,"column":20},"action":"insert","lines":["o"],"id":1027}],[{"start":{"row":63,"column":20},"end":{"row":63,"column":21},"action":"insert","lines":["f"],"id":1028}],[{"start":{"row":63,"column":21},"end":{"row":63,"column":22},"action":"insert","lines":[" "],"id":1029}],[{"start":{"row":63,"column":159},"end":{"row":63,"column":160},"action":"remove","lines":["."],"id":1030}],[{"start":{"row":63,"column":159},"end":{"row":63,"column":160},"action":"insert","lines":[","],"id":1031}],[{"start":{"row":63,"column":160},"end":{"row":63,"column":161},"action":"insert","lines":[" "],"id":1032}],[{"start":{"row":63,"column":161},"end":{"row":63,"column":162},"action":"insert","lines":["t"],"id":1033}],[{"start":{"row":63,"column":162},"end":{"row":63,"column":163},"action":"insert","lines":["h"],"id":1034}],[{"start":{"row":63,"column":163},"end":{"row":63,"column":164},"action":"insert","lines":["e"],"id":1035}],[{"start":{"row":63,"column":164},"end":{"row":63,"column":165},"action":"insert","lines":["n"],"id":1036}],[{"start":{"row":63,"column":165},"end":{"row":63,"column":166},"action":"insert","lines":[" "],"id":1037}],[{"start":{"row":63,"column":166},"end":{"row":63,"column":167},"action":"insert","lines":["r"],"id":1038}],[{"start":{"row":63,"column":167},"end":{"row":63,"column":168},"action":"insert","lines":["u"],"id":1039}],[{"start":{"row":63,"column":168},"end":{"row":63,"column":169},"action":"insert","lines":["n"],"id":1040}],[{"start":{"row":63,"column":169},"end":{"row":63,"column":170},"action":"insert","lines":[" "],"id":1041}],[{"start":{"row":63,"column":170},"end":{"row":63,"column":171},"action":"insert","lines":["e"],"id":1042}],[{"start":{"row":63,"column":171},"end":{"row":63,"column":172},"action":"insert","lines":["a"],"id":1043}],[{"start":{"row":63,"column":172},"end":{"row":63,"column":173},"action":"insert","lines":["c"],"id":1044}],[{"start":{"row":63,"column":173},"end":{"row":63,"column":174},"action":"insert","lines":["h"],"id":1045}],[{"start":{"row":63,"column":161},"end":{"row":64,"column":0},"action":"insert","lines":["",""],"id":1046}],[{"start":{"row":64,"column":13},"end":{"row":64,"column":14},"action":"insert","lines":[" "],"id":1047}],[{"start":{"row":64,"column":14},"end":{"row":64,"column":15},"action":"insert","lines":["m"],"id":1048}],[{"start":{"row":64,"column":15},"end":{"row":64,"column":16},"action":"insert","lines":["e"],"id":1049}],[{"start":{"row":64,"column":16},"end":{"row":64,"column":17},"action":"insert","lines":["m"],"id":1050}],[{"start":{"row":64,"column":17},"end":{"row":64,"column":18},"action":"insert","lines":["o"],"id":1051}],[{"start":{"row":64,"column":18},"end":{"row":64,"column":19},"action":"insert","lines":["i"],"id":1052}],[{"start":{"row":64,"column":19},"end":{"row":64,"column":20},"action":"insert","lines":["z"],"id":1053}],[{"start":{"row":64,"column":20},"end":{"row":64,"column":21},"action":"insert","lines":["a"],"id":1054}],[{"start":{"row":64,"column":21},"end":{"row":64,"column":22},"action":"insert","lines":["t"],"id":1055}],[{"start":{"row":64,"column":22},"end":{"row":64,"column":23},"action":"insert","lines":["i"],"id":1056}],[{"start":{"row":64,"column":23},"end":{"row":64,"column":24},"action":"insert","lines":["o"],"id":1057}],[{"start":{"row":64,"column":24},"end":{"row":64,"column":25},"action":"insert","lines":["n"],"id":1058}],[{"start":{"row":64,"column":25},"end":{"row":64,"column":26},"action":"insert","lines":[" "],"id":1059}],[{"start":{"row":64,"column":26},"end":{"row":64,"column":27},"action":"insert","lines":["l"],"id":1060}],[{"start":{"row":64,"column":27},"end":{"row":64,"column":28},"action":"insert","lines":["i"],"id":1061}],[{"start":{"row":64,"column":28},"end":{"row":64,"column":29},"action":"insert","lines":["b"],"id":1062}],[{"start":{"row":64,"column":29},"end":{"row":64,"column":30},"action":"insert","lines":[" "],"id":1063}],[{"start":{"row":64,"column":30},"end":{"row":64,"column":31},"action":"insert","lines":["t"],"id":1064}],[{"start":{"row":64,"column":31},"end":{"row":64,"column":32},"action":"insert","lines":["h"],"id":1065}],[{"start":{"row":64,"column":32},"end":{"row":64,"column":33},"action":"insert","lines":["r"],"id":1066}],[{"start":{"row":64,"column":33},"end":{"row":64,"column":34},"action":"insert","lines":["o"],"id":1067}],[{"start":{"row":64,"column":34},"end":{"row":64,"column":35},"action":"insert","lines":["u"],"id":1068}],[{"start":{"row":64,"column":35},"end":{"row":64,"column":36},"action":"insert","lines":["g"],"id":1069}],[{"start":{"row":64,"column":36},"end":{"row":64,"column":37},"action":"insert","lines":["h"],"id":1070}],[{"start":{"row":64,"column":37},"end":{"row":64,"column":38},"action":"insert","lines":[" "],"id":1071}],[{"start":{"row":64,"column":38},"end":{"row":64,"column":39},"action":"insert","lines":["t"],"id":1072}],[{"start":{"row":64,"column":39},"end":{"row":64,"column":40},"action":"insert","lines":["h"],"id":1073}],[{"start":{"row":64,"column":40},"end":{"row":64,"column":41},"action":"insert","lines":["e"],"id":1074}],[{"start":{"row":64,"column":41},"end":{"row":64,"column":42},"action":"insert","lines":["m"],"id":1075}],[{"start":{"row":64,"column":42},"end":{"row":64,"column":43},"action":"insert","lines":[" "],"id":1076}],[{"start":{"row":64,"column":43},"end":{"row":64,"column":44},"action":"insert","lines":["a"],"id":1077}],[{"start":{"row":64,"column":44},"end":{"row":64,"column":45},"action":"insert","lines":["n"],"id":1078}],[{"start":{"row":64,"column":45},"end":{"row":64,"column":46},"action":"insert","lines":["d"],"id":1079}],[{"start":{"row":64,"column":46},"end":{"row":64,"column":47},"action":"insert","lines":[" "],"id":1080}],[{"start":{"row":64,"column":47},"end":{"row":64,"column":48},"action":"insert","lines":["t"],"id":1081}],[{"start":{"row":64,"column":48},"end":{"row":64,"column":49},"action":"insert","lines":["i"],"id":1082}],[{"start":{"row":64,"column":49},"end":{"row":64,"column":50},"action":"insert","lines":["m"],"id":1083}],[{"start":{"row":64,"column":50},"end":{"row":64,"column":51},"action":"insert","lines":["e"],"id":1084}],[{"start":{"row":64,"column":51},"end":{"row":64,"column":52},"action":"insert","lines":[" "],"id":1085}],[{"start":{"row":64,"column":52},"end":{"row":64,"column":53},"action":"insert","lines":["t"],"id":1086}],[{"start":{"row":64,"column":53},"end":{"row":64,"column":54},"action":"insert","lines":["h"],"id":1087}],[{"start":{"row":64,"column":54},"end":{"row":64,"column":55},"action":"insert","lines":["e"],"id":1088}],[{"start":{"row":64,"column":55},"end":{"row":64,"column":56},"action":"insert","lines":[" "],"id":1089}],[{"start":{"row":64,"column":56},"end":{"row":64,"column":57},"action":"insert","lines":["r"],"id":1090}],[{"start":{"row":64,"column":57},"end":{"row":64,"column":58},"action":"insert","lines":["e"],"id":1091}],[{"start":{"row":64,"column":58},"end":{"row":64,"column":59},"action":"insert","lines":["s"],"id":1092}],[{"start":{"row":64,"column":59},"end":{"row":64,"column":60},"action":"insert","lines":["u"],"id":1093}],[{"start":{"row":64,"column":60},"end":{"row":64,"column":61},"action":"insert","lines":["l"],"id":1094}],[{"start":{"row":64,"column":61},"end":{"row":64,"column":62},"action":"insert","lines":["t"],"id":1095}],[{"start":{"row":64,"column":62},"end":{"row":64,"column":63},"action":"insert","lines":["s"],"id":1096}],[{"start":{"row":64,"column":63},"end":{"row":64,"column":64},"action":"insert","lines":["."],"id":1097}],[{"start":{"row":64,"column":64},"end":{"row":64,"column":65},"action":"insert","lines":[" "],"id":1098}],[{"start":{"row":64,"column":65},"end":{"row":65,"column":0},"action":"remove","lines":["",""],"id":1099}],[{"start":{"row":64,"column":82},"end":{"row":64,"column":91},"action":"remove","lines":["check out"],"id":1100},{"start":{"row":64,"column":82},"end":{"row":64,"column":83},"action":"insert","lines":["s"]}],[{"start":{"row":64,"column":83},"end":{"row":64,"column":84},"action":"insert","lines":["e"],"id":1101}],[{"start":{"row":64,"column":84},"end":{"row":64,"column":85},"action":"insert","lines":["e"],"id":1102}],[{"start":{"row":64,"column":65},"end":{"row":65,"column":0},"action":"insert","lines":["",""],"id":1103}],[{"start":{"row":64,"column":43},"end":{"row":64,"column":51},"action":"remove","lines":["and time"],"id":1104}],[{"start":{"row":64,"column":42},"end":{"row":64,"column":43},"action":"remove","lines":[" "],"id":1105}],[{"start":{"row":64,"column":42},"end":{"row":64,"column":43},"action":"insert","lines":[","],"id":1106}],[{"start":{"row":64,"column":43},"end":{"row":64,"column":44},"action":"insert","lines":[" "],"id":1107}],[{"start":{"row":64,"column":44},"end":{"row":64,"column":45},"action":"insert","lines":["t"],"id":1108}],[{"start":{"row":64,"column":45},"end":{"row":64,"column":46},"action":"insert","lines":["i"],"id":1109}],[{"start":{"row":64,"column":46},"end":{"row":64,"column":47},"action":"insert","lines":["m"],"id":1110}],[{"start":{"row":64,"column":47},"end":{"row":64,"column":48},"action":"insert","lines":["i"],"id":1111}],[{"start":{"row":64,"column":48},"end":{"row":64,"column":49},"action":"insert","lines":["n"],"id":1112}],[{"start":{"row":64,"column":49},"end":{"row":64,"column":50},"action":"insert","lines":["g"],"id":1113}],[{"start":{"row":65,"column":133},"end":{"row":66,"column":0},"action":"insert","lines":["",""],"id":1114}],[{"start":{"row":66,"column":0},"end":{"row":67,"column":0},"action":"insert","lines":["",""],"id":1115}],[{"start":{"row":67,"column":0},"end":{"row":67,"column":1},"action":"insert","lines":["H"],"id":1116}],[{"start":{"row":67,"column":1},"end":{"row":67,"column":2},"action":"insert","lines":["e"],"id":1117}],[{"start":{"row":67,"column":2},"end":{"row":67,"column":3},"action":"insert","lines":["r"],"id":1118}],[{"start":{"row":67,"column":3},"end":{"row":67,"column":4},"action":"insert","lines":["e"],"id":1119}],[{"start":{"row":67,"column":4},"end":{"row":67,"column":5},"action":"insert","lines":[" "],"id":1120}],[{"start":{"row":67,"column":5},"end":{"row":67,"column":6},"action":"insert","lines":["a"],"id":1121}],[{"start":{"row":67,"column":6},"end":{"row":67,"column":7},"action":"insert","lines":["r"],"id":1122}],[{"start":{"row":67,"column":7},"end":{"row":67,"column":8},"action":"insert","lines":["e"],"id":1123}],[{"start":{"row":67,"column":8},"end":{"row":67,"column":9},"action":"insert","lines":[" "],"id":1124}],[{"start":{"row":67,"column":9},"end":{"row":67,"column":10},"action":"insert","lines":["s"],"id":1125}],[{"start":{"row":67,"column":10},"end":{"row":67,"column":11},"action":"insert","lines":["o"],"id":1126}],[{"start":{"row":67,"column":11},"end":{"row":67,"column":12},"action":"insert","lines":["m"],"id":1127}],[{"start":{"row":67,"column":12},"end":{"row":67,"column":13},"action":"insert","lines":["e"],"id":1128}],[{"start":{"row":67,"column":13},"end":{"row":67,"column":14},"action":"insert","lines":[" "],"id":1129}],[{"start":{"row":67,"column":14},"end":{"row":67,"column":15},"action":"insert","lines":["r"],"id":1130}],[{"start":{"row":67,"column":15},"end":{"row":67,"column":16},"action":"insert","lines":["e"],"id":1131}],[{"start":{"row":67,"column":16},"end":{"row":67,"column":17},"action":"insert","lines":["s"],"id":1132}],[{"start":{"row":67,"column":17},"end":{"row":67,"column":18},"action":"insert","lines":["u"],"id":1133}],[{"start":{"row":67,"column":18},"end":{"row":67,"column":19},"action":"insert","lines":["l"],"id":1134}],[{"start":{"row":67,"column":19},"end":{"row":67,"column":20},"action":"insert","lines":["t"],"id":1135}],[{"start":{"row":67,"column":20},"end":{"row":67,"column":21},"action":"insert","lines":["s"],"id":1136}],[{"start":{"row":67,"column":21},"end":{"row":67,"column":22},"action":"insert","lines":[" "],"id":1137}],[{"start":{"row":67,"column":22},"end":{"row":67,"column":23},"action":"insert","lines":["o"],"id":1138}],[{"start":{"row":67,"column":23},"end":{"row":67,"column":24},"action":"insert","lines":["f"],"id":1139}],[{"start":{"row":67,"column":24},"end":{"row":67,"column":25},"action":"insert","lines":[" "],"id":1140}],[{"start":{"row":67,"column":25},"end":{"row":67,"column":26},"action":"insert","lines":["t"],"id":1141}],[{"start":{"row":67,"column":26},"end":{"row":67,"column":27},"action":"insert","lines":["h"],"id":1142}],[{"start":{"row":67,"column":27},"end":{"row":67,"column":28},"action":"insert","lines":["e"],"id":1143}],[{"start":{"row":67,"column":28},"end":{"row":67,"column":29},"action":"insert","lines":[" "],"id":1144}],[{"start":{"row":67,"column":29},"end":{"row":67,"column":30},"action":"insert","lines":["t"],"id":1145}],[{"start":{"row":67,"column":30},"end":{"row":67,"column":31},"action":"insert","lines":["o"],"id":1146}],[{"start":{"row":67,"column":31},"end":{"row":67,"column":32},"action":"insert","lines":["p"],"id":1147}],[{"start":{"row":67,"column":32},"end":{"row":67,"column":33},"action":"insert","lines":[" "],"id":1148}],[{"start":{"row":67,"column":33},"end":{"row":67,"column":34},"action":"insert","lines":["p"],"id":1149}],[{"start":{"row":67,"column":34},"end":{"row":67,"column":35},"action":"insert","lines":["r"],"id":1150}],[{"start":{"row":67,"column":35},"end":{"row":67,"column":36},"action":"insert","lines":["o"],"id":1151}],[{"start":{"row":67,"column":36},"end":{"row":67,"column":37},"action":"insert","lines":["j"],"id":1152}],[{"start":{"row":67,"column":37},"end":{"row":67,"column":38},"action":"insert","lines":["e"],"id":1153}],[{"start":{"row":67,"column":38},"end":{"row":67,"column":39},"action":"insert","lines":["c"],"id":1154}],[{"start":{"row":67,"column":39},"end":{"row":67,"column":40},"action":"insert","lines":["t"],"id":1155}],[{"start":{"row":67,"column":40},"end":{"row":67,"column":41},"action":"insert","lines":["s"],"id":1156}],[{"start":{"row":67,"column":9},"end":{"row":67,"column":13},"action":"remove","lines":["some"],"id":1157}],[{"start":{"row":67,"column":8},"end":{"row":67,"column":9},"action":"remove","lines":[" "],"id":1158}],[{"start":{"row":67,"column":20},"end":{"row":67,"column":21},"action":"insert","lines":["s"],"id":1159}],[{"start":{"row":67,"column":21},"end":{"row":67,"column":22},"action":"insert","lines":["o"],"id":1160}],[{"start":{"row":67,"column":22},"end":{"row":67,"column":23},"action":"insert","lines":["m"],"id":1161}],[{"start":{"row":67,"column":23},"end":{"row":67,"column":24},"action":"insert","lines":["e"],"id":1162}],[{"start":{"row":67,"column":24},"end":{"row":67,"column":25},"action":"insert","lines":[" "],"id":1163}],[{"start":{"row":67,"column":25},"end":{"row":67,"column":26},"action":"insert","lines":["t"],"id":1164}],[{"start":{"row":67,"column":26},"end":{"row":67,"column":27},"action":"insert","lines":["o"],"id":1165}],[{"start":{"row":67,"column":27},"end":{"row":67,"column":28},"action":"insert","lines":["p"],"id":1166}],[{"start":{"row":67,"column":28},"end":{"row":67,"column":29},"action":"insert","lines":[" "],"id":1167}],[{"start":{"row":67,"column":29},"end":{"row":67,"column":30},"action":"insert","lines":["p"],"id":1168}],[{"start":{"row":67,"column":30},"end":{"row":67,"column":31},"action":"insert","lines":["r"],"id":1169}],[{"start":{"row":67,"column":31},"end":{"row":67,"column":32},"action":"insert","lines":["o"],"id":1170}],[{"start":{"row":67,"column":32},"end":{"row":67,"column":33},"action":"insert","lines":["j"],"id":1171}],[{"start":{"row":67,"column":33},"end":{"row":67,"column":34},"action":"insert","lines":["e"],"id":1172}],[{"start":{"row":67,"column":34},"end":{"row":67,"column":35},"action":"insert","lines":["c"],"id":1173}],[{"start":{"row":67,"column":35},"end":{"row":67,"column":36},"action":"insert","lines":["t"],"id":1174}],[{"start":{"row":67,"column":36},"end":{"row":67,"column":37},"action":"insert","lines":["s"],"id":1175}],[{"start":{"row":67,"column":37},"end":{"row":67,"column":38},"action":"insert","lines":[":"],"id":1176}],[{"start":{"row":67,"column":38},"end":{"row":67,"column":54},"action":"remove","lines":["the top projects"],"id":1177}],[{"start":{"row":1,"column":36},"end":{"row":1,"column":40},"action":"remove","lines":["780b"],"id":1178},{"start":{"row":1,"column":36},"end":{"row":1,"column":37},"action":"insert","lines":["4"]}],[{"start":{"row":1,"column":37},"end":{"row":1,"column":38},"action":"insert","lines":["6"],"id":1179}],[{"start":{"row":1,"column":37},"end":{"row":1,"column":38},"action":"remove","lines":["6"],"id":1180}],[{"start":{"row":1,"column":37},"end":{"row":1,"column":38},"action":"insert","lines":["7"],"id":1181}],[{"start":{"row":1,"column":38},"end":{"row":1,"column":39},"action":"insert","lines":["6"],"id":1182}],[{"start":{"row":1,"column":39},"end":{"row":1,"column":40},"action":"insert","lines":["b"],"id":1183}],[{"start":{"row":1,"column":40},"end":{"row":1,"column":41},"action":"insert","lines":[" "],"id":1184}],[{"start":{"row":1,"column":41},"end":{"row":1,"column":42},"action":"insert","lines":["m"],"id":1185}],[{"start":{"row":1,"column":42},"end":{"row":1,"column":43},"action":"insert","lines":["i"],"id":1186}],[{"start":{"row":1,"column":43},"end":{"row":1,"column":44},"action":"insert","lines":["n"],"id":1187}],[{"start":{"row":1,"column":44},"end":{"row":1,"column":45},"action":"insert","lines":["/"],"id":1188}],[{"start":{"row":1,"column":45},"end":{"row":1,"column":46},"action":"insert","lines":["g"],"id":1189}],[{"start":{"row":1,"column":46},"end":{"row":1,"column":47},"action":"insert","lines":["z"],"id":1190}],[{"start":{"row":1,"column":47},"end":{"row":1,"column":48},"action":"insert","lines":["i"],"id":1191}],[{"start":{"row":1,"column":48},"end":{"row":1,"column":49},"action":"insert","lines":["p"],"id":1192}],[{"start":{"row":1,"column":49},"end":{"row":1,"column":50},"action":"insert","lines":["p"],"id":1193}],[{"start":{"row":1,"column":50},"end":{"row":1,"column":51},"action":"insert","lines":["e"],"id":1194}],[{"start":{"row":1,"column":51},"end":{"row":1,"column":52},"action":"insert","lines":["d"],"id":1195}],[{"start":{"row":1,"column":49},"end":{"row":1,"column":52},"action":"remove","lines":["ped"],"id":1196}],[{"start":{"row":1,"column":116},"end":{"row":1,"column":119},"action":"remove","lines":["any"],"id":1197}],[{"start":{"row":1,"column":115},"end":{"row":1,"column":116},"action":"remove","lines":[" "],"id":1198}],[{"start":{"row":1,"column":124},"end":{"row":1,"column":125},"action":"insert","lines":["s"],"id":1199}],[{"start":{"row":2,"column":5},"end":{"row":2,"column":15},"action":"remove","lines":["JavaScript"],"id":1200}],[{"start":{"row":2,"column":4},"end":{"row":2,"column":5},"action":"remove","lines":[" "],"id":1201}],[{"start":{"row":3,"column":0},"end":{"row":3,"column":41},"action":"remove","lines":["Fully supports complex object arguments. "],"id":1202}],[{"start":{"row":2,"column":0},"end":{"row":3,"column":0},"action":"insert","lines":["",""],"id":1203}],[{"start":{"row":2,"column":0},"end":{"row":2,"column":41},"action":"insert","lines":["Fully supports complex object arguments. "],"id":1204}],[{"start":{"row":5,"column":0},"end":{"row":6,"column":0},"action":"remove","lines":["Implements LRU (least recently used) cache to keep the most recently used results up to the provided limit. ",""],"id":1205},{"start":{"row":4,"column":0},"end":{"row":5,"column":0},"action":"insert","lines":["Implements LRU (least recently used) cache to keep the most recently used results up to the provided limit. ",""]}],[{"start":{"row":4,"column":0},"end":{"row":5,"column":0},"action":"remove","lines":["Implements LRU (least recently used) cache to keep the most recently used results up to the provided limit. ",""],"id":1206},{"start":{"row":3,"column":0},"end":{"row":4,"column":0},"action":"insert","lines":["Implements LRU (least recently used) cache to keep the most recently used results up to the provided limit. ",""]}],[{"start":{"row":6,"column":0},"end":{"row":7,"column":0},"action":"remove","lines":["For the browser as well as node.",""],"id":1207},{"start":{"row":5,"column":0},"end":{"row":6,"column":0},"action":"insert","lines":["For the browser as well as node.",""]}],[{"start":{"row":5,"column":0},"end":{"row":6,"column":0},"action":"remove","lines":["For the browser as well as node.",""],"id":1208},{"start":{"row":4,"column":0},"end":{"row":5,"column":0},"action":"insert","lines":["For the browser as well as node.",""]}],[{"start":{"row":7,"column":0},"end":{"row":8,"column":0},"action":"remove","lines":["",""],"id":1209}],[{"start":{"row":1,"column":36},"end":{"row":1,"column":39},"action":"remove","lines":["476"],"id":1210},{"start":{"row":1,"column":36},"end":{"row":1,"column":37},"action":"insert","lines":["9"]}],[{"start":{"row":1,"column":37},"end":{"row":1,"column":38},"action":"insert","lines":["2"],"id":1211}],[{"start":{"row":1,"column":38},"end":{"row":1,"column":39},"action":"insert","lines":["3"],"id":1212}],[{"start":{"row":2,"column":15},"end":{"row":2,"column":16},"action":"insert","lines":["m"],"id":1213}],[{"start":{"row":2,"column":16},"end":{"row":2,"column":17},"action":"insert","lines":["u"],"id":1214}],[{"start":{"row":2,"column":17},"end":{"row":2,"column":18},"action":"insert","lines":["l"],"id":1215}],[{"start":{"row":2,"column":18},"end":{"row":2,"column":19},"action":"insert","lines":["t"],"id":1216}],[{"start":{"row":2,"column":19},"end":{"row":2,"column":20},"action":"insert","lines":["i"],"id":1217}],[{"start":{"row":2,"column":20},"end":{"row":2,"column":21},"action":"insert","lines":["p"],"id":1218}],[{"start":{"row":2,"column":21},"end":{"row":2,"column":22},"action":"insert","lines":["l"],"id":1219}],[{"start":{"row":2,"column":22},"end":{"row":2,"column":23},"action":"insert","lines":["e"],"id":1220}],[{"start":{"row":2,"column":23},"end":{"row":2,"column":24},"action":"insert","lines":[" "],"id":1221}],[{"start":{"row":5,"column":97},"end":{"row":5,"column":98},"action":"insert","lines":[" "],"id":1222}],[{"start":{"row":5,"column":98},"end":{"row":5,"column":99},"action":"insert","lines":["w"],"id":1223}],[{"start":{"row":5,"column":99},"end":{"row":5,"column":100},"action":"insert","lines":["h"],"id":1224}],[{"start":{"row":5,"column":100},"end":{"row":5,"column":101},"action":"insert","lines":["e"],"id":1225}],[{"start":{"row":5,"column":101},"end":{"row":5,"column":102},"action":"insert","lines":["n"],"id":1226}],[{"start":{"row":5,"column":102},"end":{"row":5,"column":103},"action":"insert","lines":[" "],"id":1227}],[{"start":{"row":5,"column":103},"end":{"row":5,"column":104},"action":"insert","lines":["a"],"id":1228}],[{"start":{"row":5,"column":104},"end":{"row":5,"column":105},"action":"insert","lines":["v"],"id":1229}],[{"start":{"row":5,"column":105},"end":{"row":5,"column":106},"action":"insert","lines":["a"],"id":1230}],[{"start":{"row":5,"column":106},"end":{"row":5,"column":107},"action":"insert","lines":["i"],"id":1231}],[{"start":{"row":5,"column":107},"end":{"row":5,"column":108},"action":"insert","lines":["l"],"id":1232}],[{"start":{"row":5,"column":108},"end":{"row":5,"column":109},"action":"insert","lines":["a"],"id":1233}],[{"start":{"row":5,"column":109},"end":{"row":5,"column":110},"action":"insert","lines":["b"],"id":1234}],[{"start":{"row":5,"column":110},"end":{"row":5,"column":111},"action":"insert","lines":["l"],"id":1235}],[{"start":{"row":5,"column":111},"end":{"row":5,"column":112},"action":"insert","lines":["e"],"id":1236}],[{"start":{"row":5,"column":112},"end":{"row":5,"column":113},"action":"insert","lines":[" "],"id":1237}],[{"start":{"row":5,"column":113},"end":{"row":5,"column":114},"action":"insert","lines":["f"],"id":1238}],[{"start":{"row":5,"column":114},"end":{"row":5,"column":115},"action":"insert","lines":["o"],"id":1239}],[{"start":{"row":5,"column":115},"end":{"row":5,"column":116},"action":"insert","lines":["r"],"id":1240}],[{"start":{"row":5,"column":116},"end":{"row":5,"column":117},"action":"insert","lines":[" "],"id":1241}],[{"start":{"row":5,"column":117},"end":{"row":5,"column":118},"action":"insert","lines":["t"],"id":1242}],[{"start":{"row":5,"column":118},"end":{"row":5,"column":119},"action":"insert","lines":["o"],"id":1243}],[{"start":{"row":5,"column":119},"end":{"row":5,"column":120},"action":"insert","lines":["p"],"id":1244}],[{"start":{"row":5,"column":120},"end":{"row":5,"column":121},"action":"insert","lines":[" "],"id":1245}],[{"start":{"row":5,"column":121},"end":{"row":5,"column":122},"action":"insert","lines":["p"],"id":1246}],[{"start":{"row":5,"column":122},"end":{"row":5,"column":123},"action":"insert","lines":["e"],"id":1247}],[{"start":{"row":5,"column":123},"end":{"row":5,"column":124},"action":"insert","lines":["r"],"id":1248}],[{"start":{"row":5,"column":124},"end":{"row":5,"column":125},"action":"insert","lines":["f"],"id":1249}],[{"start":{"row":5,"column":125},"end":{"row":5,"column":126},"action":"insert","lines":["o"],"id":1250}],[{"start":{"row":5,"column":126},"end":{"row":5,"column":127},"action":"insert","lines":["r"],"id":1251}],[{"start":{"row":5,"column":127},"end":{"row":5,"column":128},"action":"insert","lines":["m"],"id":1252}],[{"start":{"row":5,"column":128},"end":{"row":5,"column":129},"action":"insert","lines":["a"],"id":1253}],[{"start":{"row":5,"column":129},"end":{"row":5,"column":130},"action":"insert","lines":["n"],"id":1254}],[{"start":{"row":5,"column":130},"end":{"row":5,"column":131},"action":"insert","lines":["c"],"id":1255}],[{"start":{"row":5,"column":131},"end":{"row":5,"column":132},"action":"insert","lines":["e"],"id":1256}],[{"start":{"row":5,"column":132},"end":{"row":5,"column":133},"action":"insert","lines":[","],"id":1257}],[{"start":{"row":5,"column":133},"end":{"row":6,"column":0},"action":"insert","lines":["",""],"id":1258}],[{"start":{"row":6,"column":0},"end":{"row":6,"column":1},"action":"insert","lines":["o"],"id":1259}],[{"start":{"row":6,"column":1},"end":{"row":6,"column":2},"action":"insert","lines":["t"],"id":1260}],[{"start":{"row":6,"column":2},"end":{"row":6,"column":3},"action":"insert","lines":["h"],"id":1261}],[{"start":{"row":6,"column":3},"end":{"row":6,"column":4},"action":"insert","lines":["r"],"id":1262}],[{"start":{"row":6,"column":4},"end":{"row":6,"column":5},"action":"insert","lines":["w"],"id":1263}],[{"start":{"row":6,"column":5},"end":{"row":6,"column":6},"action":"insert","lines":["i"],"id":1264}],[{"start":{"row":6,"column":6},"end":{"row":6,"column":7},"action":"insert","lines":["s"],"id":1265}],[{"start":{"row":6,"column":7},"end":{"row":6,"column":8},"action":"insert","lines":["e"],"id":1266}],[{"start":{"row":6,"column":8},"end":{"row":6,"column":9},"action":"insert","lines":[" "],"id":1267}],[{"start":{"row":6,"column":3},"end":{"row":6,"column":4},"action":"insert","lines":["e"],"id":1268}],[{"start":{"row":6,"column":9},"end":{"row":6,"column":10},"action":"remove","lines":[" "],"id":1269}],[{"start":{"row":6,"column":9},"end":{"row":6,"column":10},"action":"insert","lines":[" "],"id":1270}],[{"start":{"row":6,"column":10},"end":{"row":6,"column":11},"action":"insert","lines":["f"],"id":1271}],[{"start":{"row":6,"column":11},"end":{"row":6,"column":12},"action":"insert","lines":["a"],"id":1272}],[{"start":{"row":6,"column":12},"end":{"row":6,"column":13},"action":"insert","lines":["l"],"id":1273}],[{"start":{"row":6,"column":13},"end":{"row":6,"column":14},"action":"insert","lines":["l"],"id":1274}],[{"start":{"row":6,"column":14},"end":{"row":6,"column":15},"action":"insert","lines":["s"],"id":1275}],[{"start":{"row":6,"column":15},"end":{"row":6,"column":16},"action":"insert","lines":[" "],"id":1276}],[{"start":{"row":6,"column":16},"end":{"row":6,"column":17},"action":"insert","lines":["b"],"id":1277}],[{"start":{"row":6,"column":17},"end":{"row":6,"column":18},"action":"insert","lines":["a"],"id":1278}],[{"start":{"row":6,"column":18},"end":{"row":6,"column":19},"action":"insert","lines":["c"],"id":1279}],[{"start":{"row":6,"column":19},"end":{"row":6,"column":20},"action":"insert","lines":["k"],"id":1280}],[{"start":{"row":6,"column":20},"end":{"row":6,"column":21},"action":"insert","lines":[" "],"id":1281}],[{"start":{"row":6,"column":21},"end":{"row":6,"column":22},"action":"insert","lines":["t"],"id":1282}],[{"start":{"row":6,"column":22},"end":{"row":6,"column":23},"action":"insert","lines":["o"],"id":1283}],[{"start":{"row":6,"column":23},"end":{"row":6,"column":24},"action":"insert","lines":[" "],"id":1284}],[{"start":{"row":6,"column":24},"end":{"row":6,"column":98},"action":"remove","lines":[", compatible with: Chrome 38+, Firefox 13+, IE11+, Safari 7.1+, Opera 25+."],"id":1285}],[{"start":{"row":5,"column":0},"end":{"row":6,"column":24},"action":"remove","lines":["Uses [Map]([https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map) when available for top performance,","otherwise falls back to "],"id":1286}],[{"start":{"row":4,"column":32},"end":{"row":5,"column":0},"action":"remove","lines":["",""],"id":1287}],[{"start":{"row":1,"column":101},"end":{"row":1,"column":102},"action":"insert","lines":["m"],"id":1288}],[{"start":{"row":1,"column":102},"end":{"row":1,"column":103},"action":"insert","lines":["e"],"id":1289}],[{"start":{"row":1,"column":103},"end":{"row":1,"column":104},"action":"insert","lines":["m"],"id":1290}],[{"start":{"row":1,"column":104},"end":{"row":1,"column":105},"action":"insert","lines":["o"],"id":1291}],[{"start":{"row":1,"column":105},"end":{"row":1,"column":106},"action":"insert","lines":["i"],"id":1292}],[{"start":{"row":1,"column":106},"end":{"row":1,"column":107},"action":"insert","lines":["z"],"id":1293}],[{"start":{"row":1,"column":107},"end":{"row":1,"column":108},"action":"insert","lines":["a"],"id":1294}],[{"start":{"row":1,"column":108},"end":{"row":1,"column":109},"action":"insert","lines":["t"],"id":1295}],[{"start":{"row":1,"column":109},"end":{"row":1,"column":110},"action":"insert","lines":["i"],"id":1296}],[{"start":{"row":1,"column":110},"end":{"row":1,"column":111},"action":"insert","lines":["o"],"id":1297}],[{"start":{"row":1,"column":111},"end":{"row":1,"column":112},"action":"insert","lines":["n"],"id":1298}],[{"start":{"row":1,"column":112},"end":{"row":1,"column":113},"action":"insert","lines":[" "],"id":1299}],[{"start":{"row":1,"column":113},"end":{"row":1,"column":114},"action":"insert","lines":["l"],"id":1300}],[{"start":{"row":1,"column":114},"end":{"row":1,"column":115},"action":"insert","lines":["i"],"id":1301}],[{"start":{"row":1,"column":115},"end":{"row":1,"column":116},"action":"insert","lines":["b"],"id":1302}],[{"start":{"row":1,"column":116},"end":{"row":1,"column":117},"action":"insert","lines":["."],"id":1303}],[{"start":{"row":1,"column":117},"end":{"row":1,"column":118},"action":"insert","lines":[" "],"id":1304}],[{"start":{"row":1,"column":118},"end":{"row":1,"column":143},"action":"remove","lines":["lib to memoize functions."],"id":1305}]]},"ace":{"folds":[],"scrolltop":0,"scrollleft":0,"selection":{"start":{"row":4,"column":32},"end":{"row":4,"column":32},"isBackwards":false},"options":{"guessTabSize":true,"useWrapMode":false,"wrapToView":true},"firstLineState":0},"timestamp":1454312533247}